# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base

CONF = config.CONF


class PortSecurityTest(base.BaseTempestTestCase):
    credentials = ['primary', 'admin']
    required_extensions = ['port-security']

    @decorators.idempotent_id('61ab176e-d48b-42b7-b38a-1ba571ecc033')
    def test_port_security_removed_added(self):
        """Test connection works after port security has been removed

        Initial test that vm is accessible. Then port security is removed,
        checked connectivity. Port security is added back and checked
        connectivity again.
        """
        self.setup_network_and_server()
        self.check_connectivity(self.fip['floating_ip_address'],
                                CONF.validation.image_ssh_user,
                                self.keypair['private_key'])
        sec_group_id = self.security_groups[0]['id']

        self.port = self.update_port(port=self.port,
                                     port_security_enabled=False,
                                     security_groups=[])
        self.check_connectivity(self.fip['floating_ip_address'],
                                CONF.validation.image_ssh_user,
                                self.keypair['private_key'])

        self.port = self.update_port(port=self.port,
                                     port_security_enabled=True,
                                     security_groups=[sec_group_id])
        self.check_connectivity(self.fip['floating_ip_address'],
                                CONF.validation.image_ssh_user,
                                self.keypair['private_key'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.db import constants as db_const
from oslo_utils import uuidutils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base

LONG_NAME_NG = 'z' * (db_const.NAME_FIELD_SIZE + 1)
LONG_DESCRIPTION_NG = 'z' * (db_const.LONG_DESCRIPTION_FIELD_SIZE + 1)


class PortsNegativeTestJSON(base.BaseNetworkTest):

    @classmethod
    def resource_setup(cls):
        super(PortsNegativeTestJSON, cls).resource_setup()
        cls.network = cls.create_network()

    @decorators.attr(type='negative')
    @decorators.idempotent_id('0cbd256a-a6d4-4afa-a039-44cc13704bab')
    def test_add_port_with_too_long_name(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.create_port,
                          self.network, name=LONG_NAME_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('e10da38c-1071-49c9-95c2-0c451e18ae31')
    def test_add_port_with_too_long_description(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.create_port,
                          self.network, description=LONG_DESCRIPTION_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('5b69a905-3a84-43a4-807a-1a67ab85caeb')
    def test_add_port_with_nonexist_tenant_id(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.create_port,
                          self.network,
                          project_id=uuidutils.generate_uuid())

    @decorators.attr(type='negative')
    @decorators.idempotent_id('7cf473ae-7ec8-4834-ae17-9ef6ec6b8a32')
    def test_add_port_with_nonexist_network_id(self):
        network = self.network
        # Copy and restore net ID so the cleanup will delete correct net
        original_network_id = network['id']
        network['id'] = uuidutils.generate_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.create_port,
                          network)
        network['id'] = original_network_id

    @decorators.attr(type='negative')
    @decorators.idempotent_id('cad2d349-25fa-490e-9675-cd2ea24164bc')
    def test_add_port_with_nonexist_security_groups_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.create_port,
                          self.network,
                          security_groups=[uuidutils.generate_uuid()])

    @decorators.attr(type='negative')
    @decorators.idempotent_id('9b0a4152-9aa4-4169-9b2c-579609e2fb4a')
    def test_add_port_with_illegal_ip(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.create_port,
                          self.network,
                          allowed_address_pairs=[{"ip_address: 12.12.12.a"}])



# Copyright (c) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest.common import utils
from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base
from neutron_tempest_plugin.scenario import constants


CONF = config.CONF


class Fip64(base.BaseTempestTestCase):
    credentials = ['primary', 'admin']

    _fip_ip_version = 6

    @classmethod
    @utils.requires_ext(extension="fip64", service="network")
    def resource_setup(cls):
        super(Fip64, cls).resource_setup()
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        router = cls.create_router_by_client()
        cls.create_router_interface(router['id'], cls.subnet['id'])
        cls.keypair = cls.create_keypair()

        cls.secgroup = cls.os_primary.network_client.create_security_group(
            name=data_utils.rand_name('secgroup-'))['security_group']
        cls.security_groups.append(cls.secgroup)
        cls.create_loginable_secgroup_rule(secgroup_id=cls.secgroup['id'])

    def _find_ipv6_subnet(self, network_id):
        subnets = self.os_admin.network_client.list_subnets(
            network_id=network_id)['subnets']
        for subnet in subnets:
            if subnet['ip_version'] == self._fip_ip_version:
                return subnet['id']
        msg = "No suitable subnets on public network"
        raise self.skipException(msg)

    def _create_and_associate_floatingip64(self, port_id):
        network_id = CONF.network.public_network_id
        subnet_id = self._find_ipv6_subnet(network_id)
        fip = self.os_primary.network_client.create_floatingip(
            floating_network_id=network_id,
            subnet_id=subnet_id,
            port_id=port_id)['floatingip']
        self.floating_ips.append(fip)
        self.assertEqual(
            self._fip_ip_version,
            netaddr.IPAddress(fip['floating_ip_address']).version)
        return fip

    def _create_server_with_fip64(self):
        port = self.create_port(self.network, security_groups=[
            self.secgroup['id']])
        fip = self._create_and_associate_floatingip64(port['id'])
        server = self.create_server(
            flavor_ref=CONF.compute.flavor_ref,
            image_ref=CONF.compute.image_ref,
            key_name=self.keypair['name'],
            networks=[{'port': port['id']}])['server']
        waiters.wait_for_server_status(self.os_primary.servers_client,
                                       server['id'],
                                       constants.SERVER_STATUS_ACTIVE)
        return {'port': port, 'fip': fip, 'server': server}

    @decorators.idempotent_id('63f7da91-c7dd-449b-b50b-1c56853ce0ef')
    def test_fip64(self):
        server = self._create_server_with_fip64()
        self.check_connectivity(server['fip']['floating_ip_address'],
                                CONF.validation.image_ssh_user,
                                self.keypair['private_key'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random

from neutron_lib import constants
from oslo_log import log
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions
import testtools

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin.api import base_security_groups

LOG = log.getLogger(__name__)


ADDRESS_GROUP_NAME = 'test-address-group'


class AddressGroupTest(base.BaseAdminNetworkTest):

    credentials = ['primary', 'admin']
    required_extensions = ['address-group']

    @decorators.idempotent_id('496fef1b-22ce-483b-ab93-d28bf46954b0')
    def test_address_group_lifecycle(self):
        ag_description = "Test AG description"
        ag_name = data_utils.rand_name(ADDRESS_GROUP_NAME)
        addresses = ['10.10.10.3/32', '192.168.0.10/24', '2001:db8::f00/64']
        expected_addresses = [
            '10.10.10.3/32', '192.168.0.0/24', '2001:db8::/64']
        ag = self.create_address_group(
            description=ag_description,
            name=ag_name,
            addresses=addresses)
        self.assertEqual(ag_description, ag['description'])
        self.assertEqual(ag_name, ag['name'])
        self.assertListEqual(
            sorted(expected_addresses), sorted(ag['addresses']))

        new_description = 'New AG description'
        updated_ag = self.client.update_address_group(
            ag['id'], description=new_description)['address_group']
        self.assertEqual(new_description, updated_ag['description'])
        self.assertEqual(ag_name, updated_ag['name'])
        self.assertListEqual(
            sorted(expected_addresses), sorted(updated_ag['addresses']))

        self.client.delete_address_group(ag['id'])
        with testtools.ExpectedException(exceptions.NotFound):
            self.client.show_address_group(ag['id'])

    @decorators.idempotent_id('8a42029a-40eb-4b44-a7cf-38500046f9b8')
    def test_address_group_create_with_wrong_address(self):
        with testtools.ExpectedException(exceptions.BadRequest):
            self.create_address_group(
                name=data_utils.rand_name(ADDRESS_GROUP_NAME),
                addresses=['10.20.30.40'])

        with testtools.ExpectedException(exceptions.BadRequest):
            self.create_address_group(
                name=data_utils.rand_name(ADDRESS_GROUP_NAME),
                addresses=['this is bad IP address'])

    @decorators.idempotent_id('27c03921-bb12-4b9a-b32e-7083bc90ff1f')
    def test_edit_addresses_in_address_group(self):
        addresses = ['10.10.10.3/32', '192.168.0.10/24', '2001:db8::f00/64']
        expected_addresses = [
            '10.10.10.3/32', '192.168.0.0/24', '2001:db8::/64']
        ag = self.create_address_group(
            name=data_utils.rand_name(ADDRESS_GROUP_NAME),
            addresses=addresses)
        self.assertListEqual(
            sorted(expected_addresses), sorted(ag['addresses']))

        added_addresses = ['10.20.30.40/32']
        self.client.add_addresses_to_address_group(
            ag['id'], addresses=added_addresses)
        updated_ag = self.client.show_address_group(ag['id'])['address_group']
        expected_addresses += added_addresses
        self.assertListEqual(
            sorted(expected_addresses), sorted(updated_ag['addresses']))

        removed_addresses = [expected_addresses.pop(0)]
        self.client.remove_addresses_from_address_group(
            ag['id'], addresses=removed_addresses)
        updated_ag = self.client.show_address_group(ag['id'])['address_group']
        self.assertListEqual(
            sorted(expected_addresses), sorted(updated_ag['addresses']))

    @decorators.idempotent_id('feec6747-b4b8-49e3-8cff-817d3f097f2c')
    def test_add_wrong_address_to_address_group(self):
        addresses = ['10.10.10.3/32', '192.168.0.10/24', '2001:db8::f00/64']
        expected_addresses = [
            '10.10.10.3/32', '192.168.0.0/24', '2001:db8::/64']
        ag = self.create_address_group(
            name=data_utils.rand_name(ADDRESS_GROUP_NAME),
            addresses=addresses)
        self.assertListEqual(
            sorted(expected_addresses), sorted(ag['addresses']))
        with testtools.ExpectedException(exceptions.BadRequest):
            self.client.add_addresses_to_address_group(
                ag['id'], addresses=['this is bad IP address'])
        updated_ag = self.client.show_address_group(ag['id'])['address_group']
        self.assertListEqual(
            sorted(expected_addresses), sorted(updated_ag['addresses']))

    @decorators.idempotent_id('74f6fd4c-257b-4725-887b-470e96960e24')
    def test_remove_wrong_address_from_address_group(self):
        addresses = ['10.10.10.3/32', '192.168.0.10/24', '2001:db8::f00/64']
        expected_addresses = [
            '10.10.10.3/32', '192.168.0.0/24', '2001:db8::/64']
        ag = self.create_address_group(
            name=data_utils.rand_name(ADDRESS_GROUP_NAME),
            addresses=addresses)
        self.assertListEqual(
            sorted(expected_addresses), sorted(ag['addresses']))
        with testtools.ExpectedException(exceptions.NotFound):
            self.client.remove_addresses_from_address_group(
                ag['id'], addresses=['10.200.200.200'])
        updated_ag = self.client.show_address_group(ag['id'])['address_group']
        self.assertListEqual(
            sorted(expected_addresses), sorted(updated_ag['addresses']))

        with testtools.ExpectedException(exceptions.BadRequest):
            self.client.remove_addresses_from_address_group(
                ag['id'], addresses=['this is bad IP address'])
        updated_ag = self.client.show_address_group(ag['id'])['address_group']
        self.assertListEqual(
            sorted(expected_addresses), sorted(updated_ag['addresses']))


class RbacSharedAddressGroupTest(base.BaseAdminNetworkTest):

    force_tenant_isolation = True
    credentials = ['primary', 'alt', 'admin']
    required_extensions = ['security-group', 'address-group',
                           'rbac-address-group']

    @classmethod
    def resource_setup(cls):
        super(RbacSharedAddressGroupTest, cls).resource_setup()
        cls.client2 = cls.os_alt.network_client

    def _create_address_group(self, is_admin=False, **kwargs):
        name = data_utils.rand_name(ADDRESS_GROUP_NAME)
        return self.create_address_group(name=name, is_admin=is_admin,
                                         **kwargs)

    def _make_admin_ag_shared_to_project_id(self, project_id):
        ag = self._create_address_group(is_admin=True)
        rbac_policy = self.admin_client.create_rbac_policy(
            object_type='address_group',
            object_id=ag['id'],
            action='access_as_shared',
            target_tenant=project_id,
        )['rbac_policy']
        return {'address_group': ag, 'rbac_policy': rbac_policy}

    @decorators.idempotent_id('95f59a88-c47e-4dd9-a231-85f1782753a7')
    def test_policy_target_update(self):
        res = self._make_admin_ag_shared_to_project_id(
            self.client.tenant_id)
        # change to client2
        update_res = self.admin_client.update_rbac_policy(
                res['rbac_policy']['id'], target_tenant=self.client2.tenant_id)
        self.assertEqual(self.client2.tenant_id,
                         update_res['rbac_policy']['target_tenant'])
        # make sure everything else stayed the same
        res['rbac_policy'].pop('target_tenant')
        update_res['rbac_policy'].pop('target_tenant')
        self.assertEqual(res['rbac_policy'], update_res['rbac_policy'])

    @decorators.idempotent_id('35a214c9-5c99-468f-9242-34d0529cabfa')
    def test_secgrprule_presence_prevents_policy_rbac_policy_deletion(self):
        res = self._make_admin_ag_shared_to_project_id(
            self.client2.tenant_id)
        ag_id = res['address_group']['id']
        security_group = self.create_security_group(client=self.client2)
        protocol = random.choice(list(base_security_groups.V4_PROTOCOL_NAMES))
        sec_grp_rule = self.create_security_group_rule(
            security_group=security_group,
            client=self.client2, protocol=protocol,
            direction=constants.INGRESS_DIRECTION,
            remote_address_group_id=ag_id)

        # a port with shared sg should prevent the deletion of an
        # rbac-policy required for it to be shared
        with testtools.ExpectedException(exceptions.Conflict):
            self.admin_client.delete_rbac_policy(res['rbac_policy']['id'])

        # cleanup
        self.client2.delete_security_group_rule(sec_grp_rule['id'])
        self.admin_client.delete_rbac_policy(res['rbac_policy']['id'])

    @decorators.idempotent_id('c89db8d4-0b52-4072-ac7e-672860491843')
    def test_regular_client_shares_to_another_regular_client(self):
        # owned by self.admin_client
        ag = self._create_address_group(is_admin=True)
        with testtools.ExpectedException(exceptions.NotFound):
            self.client.show_address_group(ag['id'])
        rbac_policy = self.admin_client.create_rbac_policy(
            object_type='address_group', object_id=ag['id'],
            action='access_as_shared',
            target_tenant=self.client.tenant_id)['rbac_policy']
        self.client.show_address_group(ag['id'])

        self.assertIn(rbac_policy,
                      self.admin_client.list_rbac_policies()['rbac_policies'])
        # ensure that 'client2' can't see the rbac-policy sharing the
        # ag to it because the rbac-policy belongs to 'client'
        self.assertNotIn(rbac_policy['id'], [p['id'] for p in
                         self.client2.list_rbac_policies()['rbac_policies']])

    @decorators.idempotent_id('55a9fbb6-3333-48e8-90e4-11ab2a49567b')
    def test_filter_fields(self):
        ag = self._create_address_group()
        self.admin_client.create_rbac_policy(
            object_type='address_group', object_id=ag['id'],
            action='access_as_shared', target_tenant=self.client2.tenant_id)
        field_args = (('id',), ('id', 'action'), ('object_type', 'object_id'),
                      ('project_id', 'target_tenant'))
        for fields in field_args:
            res = self.admin_client.list_rbac_policies(fields=fields)
            self.assertEqual(set(fields), set(res['rbac_policies'][0].keys()))

    @decorators.idempotent_id('20b2706b-1cea-4724-ab72-d7452ecb1fc4')
    def test_rbac_policy_show(self):
        res = self._make_admin_ag_shared_to_project_id(
            self.client.tenant_id)
        p1 = res['rbac_policy']
        p2 = self.admin_client.create_rbac_policy(
            object_type='address_group',
            object_id=res['address_group']['id'],
            action='access_as_shared',
            target_tenant='*')['rbac_policy']

        self.assertEqual(
            p1, self.admin_client.show_rbac_policy(p1['id'])['rbac_policy'])
        self.assertEqual(
            p2, self.admin_client.show_rbac_policy(p2['id'])['rbac_policy'])

    @decorators.idempotent_id('774fc038-486c-4507-ab04-c5aac0fca5ab')
    def test_filter_rbac_policies(self):
        ag = self._create_address_group()
        rbac_pol1 = self.admin_client.create_rbac_policy(
            object_type='address_group', object_id=ag['id'],
            action='access_as_shared',
            target_tenant=self.client2.tenant_id)['rbac_policy']
        rbac_pol2 = self.admin_client.create_rbac_policy(
            object_type='address_group', object_id=ag['id'],
            action='access_as_shared',
            target_tenant=self.admin_client.tenant_id)['rbac_policy']
        res1 = self.admin_client.list_rbac_policies(id=rbac_pol1['id'])[
            'rbac_policies']
        res2 = self.admin_client.list_rbac_policies(id=rbac_pol2['id'])[
            'rbac_policies']
        self.assertEqual(1, len(res1))
        self.assertEqual(1, len(res2))
        self.assertEqual(rbac_pol1['id'], res1[0]['id'])
        self.assertEqual(rbac_pol2['id'], res2[0]['id'])

    @decorators.idempotent_id('a0f3a01a-e2c7-47d6-9385-0cd7a7f0c996')
    def test_regular_client_blocked_from_sharing_anothers_policy(self):
        ag = self._make_admin_ag_shared_to_project_id(
            self.client.tenant_id)['address_group']
        with testtools.ExpectedException(exceptions.BadRequest):
            self.client.create_rbac_policy(
                object_type='address_group', object_id=ag['id'],
                action='access_as_shared',
                target_tenant=self.client2.tenant_id)

        # make sure the rbac-policy is invisible to the tenant for which it's
        # being shared
        self.assertFalse(self.client.list_rbac_policies()['rbac_policies'])

    @decorators.idempotent_id('f39e32d9-4733-48ec-b550-07f0ec4998a9')
    def test_regular_client_blocked_from_updating_shared_address_group(self):
        # owned by self.admin_client
        ag = self._create_address_group(is_admin=True)
        self.admin_client.create_rbac_policy(
            object_type='address_group', object_id=ag['id'],
            action='access_as_shared',
            target_tenant=self.client.tenant_id)['rbac_policy']
        self.client.show_address_group(ag['id'])
        with testtools.ExpectedException(exceptions.NotFound):
            self.client.update_address_group(ag['id'], name='new_name')



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import exceptions

AGENT_TYPE = 'L3 agent'
AGENT_MODES = (
    'legacy',
    'dvr_snat'
)


class L3AgentSchedulerTestJSON(base.BaseAdminNetworkTest):
    _agent_mode = 'legacy'

    """
    Tests the following operations in the Neutron API using the REST client for
    Neutron:

        List routers that the given L3 agent is hosting.
        List L3 agents hosting the given router.
        Add and Remove Router to L3 agent

    v2.0 of the Neutron API is assumed.

    The l3_agent_scheduler extension is required for these tests.
    """

    required_extensions = ['l3_agent_scheduler']

    @classmethod
    def resource_setup(cls):
        super(L3AgentSchedulerTestJSON, cls).resource_setup()
        body = cls.admin_client.list_agents()
        agents = body['agents']
        for agent in agents:
            # TODO(armax): falling back on default _agent_mode can be
            # dropped as soon as Icehouse is dropped.
            agent_mode = (
                agent['configurations'].get('agent_mode', cls._agent_mode))
            if agent['agent_type'] == AGENT_TYPE and agent_mode in AGENT_MODES:
                cls.agent = agent
                break
        else:
            msg = "L3 Agent Scheduler enabled in conf, but L3 Agent not found"
            raise exceptions.InvalidConfiguration(msg)
        cls.router = cls.create_router(data_utils.rand_name('router'))

    @decorators.idempotent_id('b7ce6e89-e837-4ded-9b78-9ed3c9c6a45a')
    def test_list_routers_on_l3_agent(self):
        self.admin_client.list_routers_on_l3_agent(self.agent['id'])

    @decorators.idempotent_id('9464e5e7-8625-49c3-8fd1-89c52be59d66')
    def test_add_list_remove_router_on_l3_agent(self):
        l3_agent_ids = list()

        # First list agents which host router
        body = self.admin_client.list_l3_agents_hosting_router(
            self.router['id'])
        # Now remove router from all agents
        for agent in body['agents']:
            self.admin_client.remove_router_from_l3_agent(
                agent['id'], self.router['id'])

        # Now list agents which host router again - list should be empty
        body = self.admin_client.list_l3_agents_hosting_router(
            self.router['id'])
        self.assertEqual([], body['agents'])

        # Now add router to one of agents
        self.admin_client.add_router_to_l3_agent(
            self.agent['id'],
            router_id=self.router['id'])

        # And check that router is hosted by this agent
        body = self.admin_client.list_l3_agents_hosting_router(
            self.router['id'])
        for agent in body['agents']:
            l3_agent_ids.append(agent['id'])
            self.assertIn('agent_type', agent)
            self.assertEqual('L3 agent', agent['agent_type'])
        self.assertIn(self.agent['id'], l3_agent_ids)



# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr
from oslo_log import log
from oslo_utils import netutils

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.scenario import base
from neutron_tempest_plugin.tap_as_a_service.services import taas_client

CONF = config.CONF

LOG = log.getLogger(__name__)


class BaseTaasScenarioTests(base.BaseTempestTestCase):

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(BaseTaasScenarioTests, cls).setup_clients()

        cls.client = cls.os_primary.network_client
        cls.admin_network_client = cls.os_admin.network_client

        # Setup taas clients
        cls.tap_services_client = taas_client.TapServicesClient(
            cls.os_primary.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **cls.os_primary.default_params)
        cls.tap_flows_client = taas_client.TapFlowsClient(
            cls.os_primary.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **cls.os_primary.default_params)

    def _create_subnet(self, network, subnets_client=None,
                       namestart='subnet-smoke', **kwargs):
        """Create a subnet for the given network

        within the cidr block configured for tenant networks.
        """
        if not subnets_client:
            subnets_client = self.client

        def cidr_in_use(cidr, tenant_id):
            """Check cidr existence

            :returns: True if subnet with cidr already exist in tenant
                  False else
            """
            cidr_in_use = self.os_admin.network_client.list_subnets(
                tenant_id=tenant_id, cidr=cidr)['subnets']
            return len(cidr_in_use) != 0

        ip_version = kwargs.pop('ip_version', 4)

        if ip_version == 6:
            tenant_cidr = netaddr.IPNetwork(
                CONF.network.project_network_v6_cidr)
            num_bits = CONF.network.project_network_v6_mask_bits
        else:
            tenant_cidr = netaddr.IPNetwork(CONF.network.project_network_cidr)
            num_bits = CONF.network.project_network_mask_bits

        result = None
        str_cidr = None
        # Repeatedly attempt subnet creation with sequential cidr
        # blocks until an unallocated block is found.
        for subnet_cidr in tenant_cidr.subnet(num_bits):
            str_cidr = str(subnet_cidr)
            if cidr_in_use(str_cidr, tenant_id=network['tenant_id']):
                continue

            subnet = dict(
                name=data_utils.rand_name(namestart),
                network_id=network['id'],
                tenant_id=network['tenant_id'],
                cidr=str_cidr,
                ip_version=ip_version,
                **kwargs
            )
            try:
                result = subnets_client.create_subnet(**subnet)
                break
            except lib_exc.Conflict as e:
                is_overlapping_cidr = 'overlaps with another subnet' in str(e)
                if not is_overlapping_cidr:
                    raise
        assert result is not None, 'Unable to allocate tenant network'

        subnet = result['subnet']
        assert subnet['cidr'] == str_cidr

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                       subnets_client.delete_subnet, subnet['id'])

        return subnet

    def _get_server_port_id_and_ip4(self, server, ip_addr=None):
        ports = self.os_admin.network_client.list_ports(
            device_id=server['id'], fixed_ip=ip_addr)['ports']
        # A port can have more than one IP address in some cases.
        # If the network is dual-stack (IPv4 + IPv6), this port is associated
        # with 2 subnets
        p_status = ['ACTIVE']
        # NOTE(vsaienko) With Ironic, instances live on separate hardware
        # servers. Neutron does not bind ports for Ironic instances, as a
        # result the port remains in the DOWN state.
        # TODO(vsaienko) remove once bug: #1599836 is resolved.
        if getattr(CONF.service_available, 'ironic', False):
            p_status.append('DOWN')
        port_map = [(p["id"], fxip["ip_address"])
                    for p in ports
                    for fxip in p["fixed_ips"]
                    if netutils.is_valid_ipv4(fxip["ip_address"]) and
                    p['status'] in p_status]
        inactive = [p for p in ports if p['status'] != 'ACTIVE']
        if inactive:
            LOG.warning("Instance has ports that are not ACTIVE: %s", inactive)

        self.assertNotEqual(0, len(port_map),
                            "No IPv4 addresses found in: %s" % ports)
        self.assertEqual(len(port_map), 1,
                         "Found multiple IPv4 addresses: %s. "
                         "Unable to determine which port to target."
                         % port_map)
        return port_map[0]

    def _get_network_by_name(self, network_name):
        net = self.os_admin.network_client.list_networks(
            name=network_name)['networks']
        self.assertNotEqual(len(net), 0,
                            "Unable to get network by name: %s" % network_name)
        return net[0]

    def _run_in_background(self, sshclient, cmd):
        runInBg = "nohup %s 2>&1 &" % cmd
        sshclient.exec_command(runInBg)

    def create_networks(self, networks_client=None,
                        routers_client=None, subnets_client=None,
                        dns_nameservers=None, port_security_enabled=True):
        """Create a network with a subnet connected to a router.

        The baremetal driver is a special case since all nodes are
        on the same shared network.

        :param dns_nameservers: list of dns servers to send to subnet.
        :returns: network, subnet, router
        """
        if CONF.network.shared_physical_network:
            # NOTE(Shrews): This exception is for environments where tenant
            # credential isolation is available, but network separation is
            # not (the current baremetal case). Likely can be removed when
            # test account mgmt is reworked:
            # https://blueprints.launchpad.net/tempest/+spec/test-accounts
            if not CONF.compute.fixed_network_name:
                m = 'fixed_network_name must be specified in config'
                raise lib_exc.InvalidConfiguration(m)
            network = self._get_network_by_name(
                CONF.compute.fixed_network_name)
            router = None
            subnet = None
        else:
            network = self.create_network(
                client=networks_client,
                port_security_enabled=port_security_enabled)
            router = self._ensure_public_router(client=routers_client)
            subnet_kwargs = dict(network=network,
                                 subnets_client=subnets_client)
            # use explicit check because empty list is a valid option
            if dns_nameservers is not None:
                subnet_kwargs['dns_nameservers'] = dns_nameservers
            subnet = self._create_subnet(**subnet_kwargs)
            if not routers_client:
                routers_client = self.client
            router_id = router['id']
            routers_client.add_router_interface_with_subnet_id(
                router_id=router_id, subnet_id=subnet['id'])

            # save a cleanup job to remove this association between
            # router and subnet
            self.addCleanup(
                test_utils.call_and_ignore_notfound_exc,
                routers_client.remove_router_interface_with_subnet_id,
                router_id=router_id, subnet_id=subnet['id'])
        return network, subnet, router

    def _create_server_with_floatingip(self, use_taas_cloud_image=False,
                                       provider_net=False, **kwargs):
        network = self.network
        if use_taas_cloud_image:
            image = CONF.neutron_plugin_options.advanced_image_ref
            flavor = CONF.neutron_plugin_options.advanced_image_flavor_ref
        else:
            flavor = CONF.compute.flavor_ref
            image = CONF.compute.image_ref

        if provider_net:
            network = self.provider_network

        port = self.create_port(
            network=network, security_groups=[self.secgroup['id']], **kwargs)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.client.delete_port, port['id'])

        params = {
            'flavor_ref': flavor,
            'image_ref': image,
            'key_name': self.keypair['name']
        }
        vm = self.create_server(networks=[{'port': port['id']}], **params)
        self.wait_for_server_active(vm['server'])
        self.wait_for_guest_os_ready(vm['server'])

        fip = self.create_and_associate_floatingip(
            port_id=port['id'])

        return port, fip

    def _setup_provider_network(self):
        net = self._create_provider_network()
        self._create_provider_subnet(net["id"])
        return net

    def _create_provider_network(self):
        network_kwargs = {
            "admin_state_up": True,
            "shared": True,
            "provider:network_type": "vlan",
            "provider:physical_network":
                CONF.taas.provider_physical_network,
        }

        segmentation_id = CONF.taas.provider_segmentation_id
        if segmentation_id and segmentation_id == "0":
            network_kwargs['provider:network_type'] = 'flat'
        elif segmentation_id:
            network_kwargs['provider:segmentation_id'] = segmentation_id

        network = self.admin_network_client.create_network(
            **network_kwargs)['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_network_client.delete_network,
                        network['id'])

        return network

    def _create_provider_subnet(self, net_id):
        subnet = dict(
            network_id=net_id,
            cidr="172.25.100.0/24",
            ip_version=4,
        )
        result = self.admin_network_client.create_subnet(**subnet)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.admin_network_client.delete_subnet, result['subnet']['id'])

        self.admin_network_client.add_router_interface_with_subnet_id(
            self.router['id'], subnet_id=result['subnet']['id'])

        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.admin_network_client.remove_router_interface_with_subnet_id,
            self.router['id'], subnet_id=result['subnet']['id'])



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.api import base_routers as base


class RoutersTestDVRBase(base.BaseRouterTest):

    required_extensions = ['router', 'dvr']

    @classmethod
    def resource_setup(cls):
        # The check above will pass if api_extensions=all, which does
        # not mean DVR extension itself is present.
        # Instead, we have to check whether DVR is actually present by using
        # admin credentials to create router with distributed=True attribute
        # and checking for BadRequest exception and that the resulting router
        # has a distributed attribute.
        super(RoutersTestDVRBase, cls).resource_setup()
        name = data_utils.rand_name('pretest-check')
        router = cls.admin_client.create_router(name)
        if 'distributed' not in router['router']:
            msg = "'distributed' attribute not found. DVR Possibly not enabled"
            raise cls.skipException(msg)
        cls.admin_client.delete_router(router['router']['id'])


class RoutersTestDVR(RoutersTestDVRBase):

    @decorators.idempotent_id('08a2a0a8-f1e4-4b34-8e30-e522e836c44e')
    def test_distributed_router_creation(self):
        """Test distributed router creation

        Test uses administrative credentials to creates a
        DVR (Distributed Virtual Routing) router using the
        distributed=True.

        Acceptance
        The router is created and the "distributed" attribute is
        set to True
        """
        name = data_utils.rand_name('router')
        router = self._create_admin_router(name, distributed=True)
        self.assertTrue(router['distributed'])

    @decorators.idempotent_id('8a0a72b4-7290-4677-afeb-b4ffe37bc352')
    def test_centralized_router_creation(self):
        """Test centralized router creation

        Test uses administrative credentials to creates a
        CVR (Centralized Virtual Routing) router using the
        distributed=False.

        Acceptance
        The router is created and the "distributed" attribute is
        set to False, thus making it a "Centralized Virtual Router"
        as opposed to a "Distributed Virtual Router"
        """
        name = data_utils.rand_name('router')
        router = self._create_admin_router(name, distributed=False)
        self.assertFalse(router['distributed'])


class RouterTestCentralizedToDVR(RoutersTestDVRBase):

    required_extensions = ['l3-ha']

    @decorators.idempotent_id('acd43596-c1fb-439d-ada8-31ad48ae3c2e')
    def test_centralized_router_update_to_dvr(self):
        """Test centralized to DVR router update

        Test uses administrative credentials to creates a
        CVR (Centralized Virtual Routing) router using the
        distributed=False.Then it will "update" the router
        distributed attribute to True

        Acceptance
        The router is created and the "distributed" attribute is
        set to False. Once the router is updated, the distributed
        attribute will be set to True
        """
        name = data_utils.rand_name('router')
        # router needs to be in admin state down in order to be upgraded to DVR
        router = self._create_admin_router(name, distributed=False,
                                           ha=False, admin_state_up=False)
        self.assertFalse(router['distributed'])
        self.assertFalse(router['ha'])
        router = self.admin_client.update_router(router['id'],
                                                 distributed=True)
        self.assertTrue(router['router']['distributed'])



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import netaddr
from neutron_lib import constants
from oslo_log import log
from paramiko import ssh_exception as ssh_exc
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools

from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin.common import utils as neutron_utils
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base

CONF = config.CONF
LOG = log.getLogger(__name__)


class DHCPTest(base.BaseTempestTestCase):

    credentials = ['primary', 'admin']
    force_tenant_isolation = False

    @classmethod
    def resource_setup(cls):
        super(DHCPTest, cls).resource_setup()
        cls.rand_name = data_utils.rand_name(
            cls.__name__.rsplit('.', 1)[-1])
        cls.network = cls.create_network(name=cls.rand_name)
        cls.subnet = cls.create_subnet(
            network=cls.network, name=cls.rand_name)
        cls.router = cls.create_router_by_client()
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.keypair = cls.create_keypair(name=cls.rand_name)
        cls.security_group = cls.create_security_group(name=cls.rand_name)
        cls.create_loginable_secgroup_rule(cls.security_group['id'])

    @utils.requires_ext(extension='extra_dhcp_opt', service='network')
    @decorators.idempotent_id('58f7c094-1980-4e03-b0d3-6c4dd27217b1')
    def test_extra_dhcp_opts(self):
        """This test case tests DHCP extra options configured for Neutron port.

        Test is checking just extra option "15" which is domain-name
        according to the RFC 2132:
        https://tools.ietf.org/html/rfc2132#section-5.3

        To test that option, there is spawned VM connected to the port with
        configured extra_dhcp_opts and test asserts that search domain name is
        configured inside VM in /etc/resolv.conf file
        """

        test_domain = "test.domain"
        extra_dhcp_opts = [
            {'opt_name': 'domain-name',
             'opt_value': '"%s"' % test_domain}]
        port = self.create_port(
            network=self.network, name=self.rand_name,
            security_groups=[self.security_group['id']],
            extra_dhcp_opts=extra_dhcp_opts)
        floating_ip = self.create_floatingip(port=port)

        server = self.create_server(
            flavor_ref=CONF.compute.flavor_ref,
            image_ref=CONF.compute.image_ref,
            key_name=self.keypair['name'],
            networks=[{'port': port['id']}])
        self.wait_for_server_active(server['server'])
        self.wait_for_guest_os_ready(server['server'])

        try:
            ssh_client = ssh.Client(
                floating_ip['floating_ip_address'],
                CONF.validation.image_ssh_user,
                pkey=self.keypair['private_key'])
            vm_resolv_conf = ssh_client.exec_command(
                "cat /etc/resolv.conf")
            self.assertIn(test_domain, vm_resolv_conf)
        except (lib_exc.SSHTimeout,
                ssh_exc.AuthenticationException,
                AssertionError) as error:
            LOG.debug(error)
            self._log_console_output([server])
            self._log_local_network_status()
            raise


class DHCPPortUpdateTest(base.BaseTempestTestCase):

    credentials = ['primary', 'admin']

    @classmethod
    def resource_setup(cls):
        super(DHCPPortUpdateTest, cls).resource_setup()
        cls.rand_name = data_utils.rand_name(
            cls.__name__.rsplit('.', 1)[-1])
        cls.network = cls.create_network(name=cls.rand_name)
        cls.router = cls.create_router_by_client()
        cls.keypair = cls.create_keypair(name=cls.rand_name)
        cls.security_group = cls.create_security_group(name=cls.rand_name)
        cls.create_loginable_secgroup_rule(cls.security_group['id'])
        cls.create_pingable_secgroup_rule(cls.security_group['id'])

    @testtools.skipUnless(
        CONF.neutron_plugin_options.firewall_driver == 'ovn',
        "OVN driver is required to run this test - "
        "LP#1942794 solution only applied to OVN")
    @decorators.idempotent_id('8171cc68-9dbb-46ca-b065-17b5b2e26094')
    def test_modify_dhcp_port_ip_address(self):
        """Test Scenario

        1) Create a network and a subnet with DHCP enabled
        2) Modify the default IP address from the subnet DHCP port
        3) Create a server in this network and check ssh connectivity

        For the step 3), the server needs to obtain ssh keys from the metadata

        Related bug: LP#1942794
        """
        # create subnet (dhcp is enabled by default)
        subnet = self.create_subnet(network=self.network, name=self.rand_name)

        def _get_dhcp_ports():
            # in some cases, like ML2/OVS, the subnet port associated to DHCP
            # is created with device_owner='network:dhcp'
            dhcp_ports = self.client.list_ports(
                network_id=self.network['id'],
                device_owner=constants.DEVICE_OWNER_DHCP)['ports']
            # in other cases, like ML2/OVN, the subnet port used for metadata
            # is created with device_owner='network:distributed'
            distributed_ports = self.client.list_ports(
                network_id=self.network['id'],
                device_owner=constants.DEVICE_OWNER_DISTRIBUTED)['ports']
            self.dhcp_ports = dhcp_ports + distributed_ports
            self.assertLessEqual(
                len(self.dhcp_ports), 1, msg='Only one port was expected')
            return len(self.dhcp_ports) == 1

        # obtain the dhcp port
        # in some cases this port is not created together with the subnet, but
        # immediately after it, so some delay may be needed and that is the
        # reason why a waiter function is used here
        self.dhcp_ports = []
        neutron_utils.wait_until_true(
            lambda: _get_dhcp_ports(),
            timeout=10)
        dhcp_port = self.dhcp_ports[0]

        # modify DHCP port IP address
        old_dhcp_port_ip = netaddr.IPAddress(
            dhcp_port['fixed_ips'][0]['ip_address'])
        if str(old_dhcp_port_ip) != subnet['allocation_pools'][0]['end']:
            new_dhcp_port_ip = str(old_dhcp_port_ip + 1)
        else:
            new_dhcp_port_ip = str(old_dhcp_port_ip - 1)
        self.update_port(port=dhcp_port,
                         fixed_ips=[{'subnet_id': subnet['id'],
                                     'ip_address': new_dhcp_port_ip}])

        # create server
        server = self.create_server(
            flavor_ref=CONF.compute.flavor_ref,
            image_ref=CONF.compute.image_ref,
            key_name=self.keypair['name'],
            security_groups=[{'name': self.security_group['name']}],
            networks=[{'uuid': self.network['id']}])

        # attach fip to the server
        self.create_router_interface(self.router['id'], subnet['id'])
        server_port = self.client.list_ports(
            network_id=self.network['id'],
            device_id=server['server']['id'])['ports'][0]
        fip = self.create_floatingip(port_id=server_port['id'])

        # check connectivity
        self.check_connectivity(fip['floating_ip_address'],
                                CONF.validation.image_ssh_user,
                                self.keypair['private_key'])



# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.scenario import manager

CONF = config.CONF

LOG = log.getLogger(__name__)


class ScenarioTest(manager.NetworkScenarioTest):
    """Base class for scenario tests. Uses tempest own clients. """

    credentials = ['primary']

    @classmethod
    def skip_checks(cls):
        super(ScenarioTest, cls).skip_checks()
        msg = None
        if not CONF.fwaas.run_fwaas_tests:
            msg = ("Running of fwaas related tests is disabled in "
                   "plugin configuration.")
        if msg:
            raise cls.skipException(msg)


class NetworkScenarioTest(ScenarioTest):
    """Base class for network scenario tests.

    This class provide helpers for network scenario tests, using the neutron
    API. Helpers from ancestor which use the nova network API are overridden
    with the neutron API.

    This Class also enforces using Neutron instead of novanetwork.
    Subclassed tests will be skipped if Neutron is not enabled

    """

    credentials = ['primary', 'admin']

    @classmethod
    def skip_checks(cls):
        super(NetworkScenarioTest, cls).skip_checks()
        if not CONF.service_available.neutron:
            raise cls.skipException('Neutron not available')

    def _create_router(self, client=None, tenant_id=None,
                       namestart='router-smoke'):
        if not client:
            client = self.routers_client
        if not tenant_id:
            tenant_id = client.tenant_id
        name = data_utils.rand_name(namestart)
        result = client.create_router(name=name,
                                      admin_state_up=True,
                                      tenant_id=tenant_id)
        router = result['router']
        self.assertEqual(router['name'], name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        client.delete_router,
                        router['id'])
        return router



# Copyright (c) 2016 Juniper Networks
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import testscenarios

from oslo_log import log as logging
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.fwaas.scenario import fwaas_v2_base as base


CONF = config.CONF
LOG = logging.getLogger(__name__)
load_tests = testscenarios.load_tests_apply_scenarios


class TestFWaaS_v2(base.FWaaSScenarioTest_V2):

    """Config Requirement in tempest.conf:

    - project_network_cidr_bits- specifies the subnet range for each network
    - project_network_cidr
    - public_network_id
    """

    def setUp(self):
        LOG.debug("Initializing FWaaSScenarioTest Setup")
        super(TestFWaaS_v2, self).setUp()
        required_exts = ['fwaas_v2', 'security-group', 'router']
        # if self.router_insertion:
        #    required_exts.append('fwaasrouterinsertion')
        for ext in required_exts:
            if not utils.is_extension_enabled(ext, 'network'):
                msg = "%s Extension not enabled." % ext
                raise self.skipException(msg)
        LOG.debug("FWaaSScenarioTest Setup done.")

    def _create_server(self, network, security_group=None):
        keys = self.create_keypair()
        kwargs = {}
        if security_group is not None:
            kwargs['security_groups'] = [{'name': security_group['name']}]
        server = self.create_server(
            key_name=keys['name'],
            networks=[{'uuid': network['id']}],
            wait_until='ACTIVE',
            **kwargs)
        return server, keys

    def _check_connectivity_between_internal_networks(
            self, floating_ip1, keys1, network2, server2, should_connect=True):
        internal_ips = (p['fixed_ips'][0]['ip_address'] for p in
                        self.os_admin.ports_client.list_ports(
                            tenant_id=server2['tenant_id'],
                            network_id=network2['id'])['ports']
                        if p['device_owner'].startswith('network'))
        self._check_server_connectivity(
            floating_ip1, keys1, internal_ips, should_connect)

    def _check_server_connectivity(self, floating_ip, keys1, address_list,
                                   should_connect=True):
        ip_address = floating_ip['floating_ip_address']
        private_key = keys1
        ssh_source = self.get_remote_client(
            ip_address, private_key=private_key)

        for remote_ip in address_list:
            if should_connect:
                msg = ("Timed out waiting for %s to become "
                       "reachable") % remote_ip
            else:
                msg = "ip address %s is reachable" % remote_ip
            try:
                self.assertTrue(self._check_remote_connectivity
                                (ssh_source, remote_ip, should_connect),
                                msg)
            except Exception:
                LOG.exception("Unable to access {dest} via ssh to "
                              "floating-ip {src}".format(dest=remote_ip,
                                                         src=floating_ip))
                raise

    def _check_remote_connectivity(self, source, dest, should_succeed=True,
                                   nic=None):
        """check ping server via source ssh connection

        :param source: RemoteClient: an ssh connection from which to ping
        :param dest: and IP to ping against
        :param should_succeed: boolean should ping succeed or not
        :param nic: specific network interface to ping from
        :returns: boolean -- should_succeed == ping
        :returns: ping is false if ping failed
        """
        def ping_remote():
            try:
                source.ping_host(dest, nic=nic)
            except lib_exc.SSHExecCommandFailed:
                LOG.warning('Failed to ping IP: %s via a ssh connection '
                            'from: %s.', dest, source.ssh_client.host)
                return not should_succeed
            return should_succeed

        return test_utils.call_until_true(ping_remote,
                                          CONF.validation.ping_timeout,
                                          1)

    def _add_router_interface(self, router_id, subnet_id):
        resp = self.routers_client.add_router_interface(
            router_id, subnet_id=subnet_id)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.routers_client.remove_router_interface, router_id,
                        subnet_id=subnet_id)
        return resp

    def _create_network_subnet(self):
        network = self.create_network()
        subnet_kwargs = dict(network=network)
        subnet = self.create_subnet(**subnet_kwargs)
        return network, subnet

    def _create_test_server(self, network, security_group):
        pub_network_id = CONF.network.public_network_id
        server, keys = self._create_server(
            network, security_group=security_group)
        private_key = keys['private_key']
        server_floating_ip = self.create_floating_ip(server, pub_network_id)
        fixed_ip = list(server['addresses'].values())[0][0]['addr']
        return server, private_key, fixed_ip, server_floating_ip

    def _create_topology(self):
        """Topology diagram:

        +--------+             +-------------+
        |"server"|             | "subnet"    |
        |   VM-1 +-------------+ "network-1" |
        +--------+             +----+--------+
                                    |
                                    | router interface port
                               +----+-----+
                               | "router" |
                               +----+-----+
                                    | router interface port
                                    |
                                    |
        +--------+             +-------------+
        |"server"|             | "subnet"    |
        |   VM-2 +-------------+ "network-2" |
        +--------+             +----+--------+
        """

        LOG.debug('Starting Topology Creation')
        resp = {}
        # Create Network1 and Subnet1.
        network1, subnet1 = self._create_network_subnet()
        resp['network1'] = network1
        resp['subnet1'] = subnet1

        # Create Network2 and Subnet2.
        network2, subnet2 = self._create_network_subnet()
        resp['network2'] = network2
        resp['subnet2'] = subnet2

        # Create a router and attach Network1, Network2 and External Networks
        # to it.
        router = self._create_router(namestart='SCENARIO-TEST-ROUTER')
        pub_network_id = CONF.network.public_network_id
        kwargs = {'external_gateway_info': dict(network_id=pub_network_id)}
        router = self.routers_client.update_router(
            router['id'], **kwargs)['router']
        router_id = router['id']
        resp_add_intf = self._add_router_interface(
            router_id, subnet_id=subnet1['id'])
        router_portid_1 = resp_add_intf['port_id']
        resp_add_intf = self._add_router_interface(
            router_id, subnet_id=subnet2['id'])
        router_portid_2 = resp_add_intf['port_id']
        resp['router'] = router
        resp['router_portid_1'] = router_portid_1
        resp['router_portid_2'] = router_portid_2

        # Create a VM on each of the network and assign it a floating IP.
        security_group = self.create_security_group()
        server1, private_key1, server_fixed_ip_1, server_floating_ip_1 = (
            self._create_test_server(network1, security_group))
        server2, private_key2, server_fixed_ip_2, server_floating_ip_2 = (
            self._create_test_server(network2, security_group))
        resp['server1'] = server1
        resp['private_key1'] = private_key1
        resp['server_fixed_ip_1'] = server_fixed_ip_1
        resp['server_floating_ip_1'] = server_floating_ip_1
        resp['server2'] = server2
        resp['private_key2'] = private_key2
        resp['server_fixed_ip_2'] = server_fixed_ip_2
        resp['server_floating_ip_2'] = server_floating_ip_2

        return resp

    @decorators.idempotent_id('77fdf3ea-82c1-453d-bfec-f7efe335625d')
    def test_icmp_reachability_scenarios(self):
        topology = self._create_topology()
        ssh_login = CONF.validation.image_ssh_user

        self.check_vm_connectivity(
            ip_address=topology['server_floating_ip_1']['floating_ip_address'],
            username=ssh_login,
            private_key=topology['private_key1'])
        self.check_vm_connectivity(
            ip_address=topology['server_floating_ip_2']['floating_ip_address'],
            username=ssh_login,
            private_key=topology['private_key2'])

        # Scenario 1: Add allow ICMP rules between the two VMs.
        fw_allow_icmp_rule = self.create_firewall_rule(action="allow",
                                                       protocol="icmp")
        fw_allow_ssh_rule = self.create_firewall_rule(action="allow",
                                                      protocol="tcp",
                                                      destination_port=22)
        fw_policy = self.create_firewall_policy(
            firewall_rules=[fw_allow_icmp_rule['id'], fw_allow_ssh_rule['id']])
        fw_group = self.create_firewall_group(
            ports=[
                topology['router_portid_1'],
                topology['router_portid_2']],
            ingress_firewall_policy_id=fw_policy['id'],
            egress_firewall_policy_id=fw_policy['id'])
        self._wait_firewall_group_ready(fw_group['id'])
        LOG.debug('fw_allow_icmp_rule: %s\nfw_allow_ssh_rule: %s\n'
                  'fw_policy: %s\nfw_group: %s\n',
                  fw_allow_icmp_rule, fw_allow_ssh_rule, fw_policy, fw_group)

        # Check the connectivity between VM1 and VM2. It should Pass.
        self._check_server_connectivity(
            topology['server_floating_ip_1'],
            topology['private_key1'],
            address_list=[topology['server_fixed_ip_2']],
            should_connect=True)

        # Scenario 2: Now remove the allow_icmp rule add a deny_icmp rule and
        # check that ICMP gets blocked
        fw_deny_icmp_rule = self.create_firewall_rule(action="deny",
                                                      protocol="icmp")
        self.remove_firewall_rule_from_policy_and_wait(
            firewall_group_id=fw_group['id'],
            firewall_rule_id=fw_allow_icmp_rule['id'],
            firewall_policy_id=fw_policy['id'])
        self.insert_firewall_rule_in_policy_and_wait(
            firewall_group_id=fw_group['id'],
            firewall_rule_id=fw_deny_icmp_rule['id'],
            firewall_policy_id=fw_policy['id'])
        self._check_server_connectivity(
            topology['server_floating_ip_1'],
            topology['private_key1'],
            address_list=[topology['server_fixed_ip_2']],
            should_connect=False)

        # Scenario 3: Create a rule allowing ICMP only from server_fixed_ip_1
        # to server_fixed_ip_2 and check that traffic from opposite direction
        # is blocked.
        fw_allow_unidirectional_icmp_rule = self.create_firewall_rule(
            action="allow", protocol="icmp",
            source_ip_address=topology['server_fixed_ip_1'],
            destination_ip_address=topology['server_fixed_ip_2'])

        self.remove_firewall_rule_from_policy_and_wait(
            firewall_group_id=fw_group['id'],
            firewall_rule_id=fw_deny_icmp_rule['id'],
            firewall_policy_id=fw_policy['id'])
        self.insert_firewall_rule_in_policy_and_wait(
            firewall_group_id=fw_group['id'],
            firewall_rule_id=fw_allow_unidirectional_icmp_rule['id'],
            firewall_policy_id=fw_policy['id'])

        self._check_server_connectivity(
            topology['server_floating_ip_1'],
            topology['private_key1'],
            address_list=[topology['server_fixed_ip_2']],
            should_connect=True)
        self._check_server_connectivity(
            topology['server_floating_ip_2'],
            topology['private_key2'],
            address_list=[topology['server_fixed_ip_1']],
            should_connect=False)

        # Disassociate ports of this firewall group for cleanup resources
        self.update_firewall_group_and_wait(fw_group['id'], ports=[])



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from neutron_tempest_plugin.api import test_trunk


class TestTrunkDetailsJSON(test_trunk.TrunkTestJSONBase):

    required_extensions = ['trunk-details']

    @decorators.idempotent_id('f0bed24f-d36a-498b-b4e7-0d66e3fb7308')
    def test_port_resource_trunk_details_no_subports(self):
        trunk = self._create_trunk_with_network_and_parent()
        parent_port = self.client.show_port(trunk['port_id'])['port']
        observed_trunk_details = parent_port.get('trunk_details')
        expected_trunk_details = {'sub_ports': [],
                                  'trunk_id': trunk['id']}
        self.assertIsNotNone(observed_trunk_details)
        self.assertEqual(expected_trunk_details, observed_trunk_details)

    @decorators.idempotent_id('544bcaf2-86fb-4930-93ab-ece1c3cc33df')
    def test_port_resource_trunk_details_with_subport(self):
        subport_network = self.create_network()
        subport = self.create_port(subport_network)
        subport_data = {'port_id': subport['id'],
                        'segmentation_type': 'vlan',
                        'segmentation_id': 2}
        trunk = self._create_trunk_with_network_and_parent([subport_data])
        parent_port = self.client.show_port(trunk['port_id'])['port']
        observed_trunk_details = parent_port.get('trunk_details')
        expected_subport_data = dict(subport_data,
                                     mac_address=subport['mac_address'])
        expected_trunk_details = {'sub_ports': [expected_subport_data],
                                  'trunk_id': trunk['id']}
        self.assertIsNotNone(observed_trunk_details)
        self.assertEqual(expected_trunk_details, observed_trunk_details)

    @decorators.idempotent_id('fe6d865f-1d5c-432e-b65d-904157172f24')
    def test_port_resource_empty_trunk_details(self):
        network = self.create_network()
        port = self.create_port(network)
        observed_port = self.client.show_port(port['id'])['port']
        observed_trunk_details = observed_port.get('trunk_details')
        self.assertIsNone(observed_trunk_details)



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
from tempest.lib.common import rest_client

from neutron_tempest_plugin.api import clients as manager
from neutron_tempest_plugin import config


CONF = config.CONF


class Manager(manager.Manager):
    def __init__(self, credentials=None, service=None):
        super(Manager, self).__init__(credentials, service)
        self.bgp_client = BgpSpeakerClientJSON(
            self.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **self.default_params)


class BgpSpeakerClientJSON(rest_client.RestClient):

    def create_bgp_speaker(self, post_data):
        post_body = jsonutils.dumps(post_data)
        resp, body = self.post('v2.0/bgp-speakers', post_body)
        body = jsonutils.loads(body)
        self.expected_success(201, resp.status)
        return rest_client.ResponseBody(resp, body)

    def get_bgp_speaker(self, id):
        uri = 'v2.0/bgp-speakers/{0}'.format(id)
        resp, body = self.get(uri)
        body = jsonutils.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def get_bgp_speakers(self):
        uri = self.get_uri("bgp-speakers")
        resp, body = self.get(uri)
        body = jsonutils.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBodyList(resp, body)

    def update_bgp_speaker(self, id, put_data):
        uri = 'v2.0/bgp-speakers/{0}'.format(id)
        update_body = {'bgp_speaker': put_data}
        update_body = jsonutils.dumps(update_body)
        resp, body = self.put(uri, update_body)
        body = jsonutils.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_bgp_speaker(self, id):
        uri = 'v2.0/bgp-speakers/{0}'.format(id)
        resp, body = self.delete(uri)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_bgp_peer(self, post_data):
        post_body = jsonutils.dumps(post_data)
        resp, body = self.post('v2.0/bgp-peers', post_body)
        body = jsonutils.loads(body)
        self.expected_success(201, resp.status)
        return rest_client.ResponseBody(resp, body)

    def get_bgp_peer(self, id):
        uri = 'v2.0/bgp-peers/{0}'.format(id)
        resp, body = self.get(uri)
        body = jsonutils.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_bgp_peer(self, id):
        uri = 'v2.0/bgp-peers/{0}'.format(id)
        resp, body = self.delete(uri)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def add_bgp_peer_with_id(self, bgp_speaker_id, bgp_peer_id):
        uri = 'v2.0/bgp-speakers/%s/add_bgp_peer' % bgp_speaker_id
        update_body = {"bgp_peer_id": bgp_peer_id}
        update_body = jsonutils.dumps(update_body)
        resp, body = self.put(uri, update_body)
        self.expected_success(200, resp.status)
        body = jsonutils.loads(body)
        return rest_client.ResponseBody(resp, body)

    def remove_bgp_peer_with_id(self, bgp_speaker_id, bgp_peer_id):
        uri = 'v2.0/bgp-speakers/%s/remove_bgp_peer' % bgp_speaker_id
        update_body = {"bgp_peer_id": bgp_peer_id}
        update_body = jsonutils.dumps(update_body)
        resp, body = self.put(uri, update_body)
        self.expected_success(200, resp.status)
        body = jsonutils.loads(body)
        return rest_client.ResponseBody(resp, body)

    def add_bgp_gateway_network(self, bgp_speaker_id, network_id):
        uri = 'v2.0/bgp-speakers/%s/add_gateway_network' % bgp_speaker_id
        update_body = {"network_id": network_id}
        update_body = jsonutils.dumps(update_body)
        resp, body = self.put(uri, update_body)
        self.expected_success(200, resp.status)
        body = jsonutils.loads(body)
        return rest_client.ResponseBody(resp, body)

    def remove_bgp_gateway_network(self, bgp_speaker_id, network_id):
        uri = 'v2.0/bgp-speakers/%s/remove_gateway_network' % bgp_speaker_id
        update_body = {"network_id": network_id}
        update_body = jsonutils.dumps(update_body)
        resp, body = self.put(uri, update_body)
        self.expected_success(200, resp.status)
        body = jsonutils.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_bgp_advertised_routes(self, bgp_speaker_id):
        base_uri = 'v2.0/bgp-speakers/%s/get_advertised_routes'
        uri = base_uri % bgp_speaker_id
        resp, body = self.get(uri)
        body = jsonutils.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_dragents_for_bgp_speaker(self, bgp_speaker_id):
        uri = 'v2.0/bgp-speakers/%s/bgp-dragents' % bgp_speaker_id
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        body = jsonutils.loads(body)
        return rest_client.ResponseBody(resp, body)

    def add_bgp_speaker_to_dragent(self, agent_id, bgp_speaker_id,
                                   ignore_errors=False):
        uri = 'v2.0/agents/%s/bgp-drinstances' % agent_id
        update_body = {"bgp_speaker_id": bgp_speaker_id}
        update_body = jsonutils.dumps(update_body)
        resp, body = self.post(uri, update_body)
        if not ignore_errors:
            self.expected_success(201, resp.status)
        body = jsonutils.loads(body)
        return rest_client.ResponseBody(resp, body)

    def remove_bgp_speaker_from_dragent(self, agent_id, bgp_speaker_id):
        uri = 'v2.0/agents/%s/bgp-drinstances/%s' % (agent_id, bgp_speaker_id)
        resp, body = self.delete(uri)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.plugins import constants
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools

from neutron_tempest_plugin.api import base_routers as base


class RoutersFlavorTestCase(base.BaseRouterTest):

    required_extensions = ['router', 'flavors', 'l3-flavors']

    @classmethod
    def resource_setup(cls):
        super(RoutersFlavorTestCase, cls).resource_setup()
        cls.service_profiles = []
        cls.flavor_service_profiles = []
        # make a flavor based on legacy router for regular tenant to use
        driver = ('neutron.services.l3_router.service_providers.'
                  'single_node.SingleNodeDriver')
        try:
            sp = cls.admin_client.create_service_profile(driver=driver)
        except lib_exc.NotFound as e:
            if e.resp_body['type'] == 'ServiceProfileDriverNotFound':
                raise cls.skipException("%s is not available" % driver)
            raise
        cls.service_profiles.append(sp['service_profile'])
        cls.flavor = cls.create_flavor(
                name='special_flavor',
                description='econonomy class',
                service_type=constants.L3)
        cls.admin_client.create_flavor_service_profile(
            cls.flavor['id'], sp['service_profile']['id'])
        cls.flavor_service_profiles.append((cls.flavor['id'],
                                            sp['service_profile']['id']))
        # make another with a different driver
        driver = ('neutron.services.l3_router.service_providers.'
                  'dvr.DvrDriver')
        try:
            sp = cls.admin_client.create_service_profile(driver=driver)
        except lib_exc.NotFound as e:
            if e.resp_body['type'] == 'ServiceProfileDriverNotFound':
                raise cls.skipException("%s is not available" % driver)
            raise
        cls.service_profiles.append(sp['service_profile'])
        cls.prem_flavor = cls.create_flavor(
                name='better_special_flavor',
                description='econonomy comfort',
                service_type=constants.L3)
        cls.admin_client.create_flavor_service_profile(
            cls.prem_flavor['id'], sp['service_profile']['id'])
        cls.flavor_service_profiles.append((cls.prem_flavor['id'],
                                            sp['service_profile']['id']))

    @classmethod
    def resource_cleanup(cls):
        for flavor_id, service_profile_id in cls.flavor_service_profiles:
            cls.admin_client.delete_flavor_service_profile(flavor_id,
                                                           service_profile_id)
        for service_profile in cls.service_profiles:
            cls.admin_client.delete_service_profile(
                service_profile['id'])
        super(RoutersFlavorTestCase, cls).resource_cleanup()

    @decorators.idempotent_id('a4d01977-e968-4983-b4d9-824ea6c33f4b')
    def test_create_router_with_flavor(self):
        # ensure regular client can see flavor
        flavors = self.client.list_flavors(id=self.flavor['id'])
        flavor = flavors['flavors'][0]
        self.assertEqual('special_flavor', flavor['name'])
        flavors = self.client.list_flavors(id=self.prem_flavor['id'])
        prem_flavor = flavors['flavors'][0]
        self.assertEqual('better_special_flavor', prem_flavor['name'])

        # ensure client can create router with both flavors
        router = self.create_router('name', flavor_id=flavor['id'])
        self.assertEqual(flavor['id'], router['flavor_id'])
        router = self.create_router('name', flavor_id=prem_flavor['id'])
        self.assertEqual(prem_flavor['id'], router['flavor_id'])

    @decorators.idempotent_id('30e73858-a0fc-409c-a2e0-e9cd2826f6a2')
    def test_delete_router_flavor_in_use(self):
        self.create_router('name', flavor_id=self.flavor['id'])
        with testtools.ExpectedException(lib_exc.Conflict):
            self.admin_client.delete_flavor(self.flavor['id'])

    @decorators.idempotent_id('83939cf7-5070-41bc-9a3e-cd9f22df2186')
    def test_badrequest_on_requesting_flags_and_flavor(self):
        with testtools.ExpectedException(lib_exc.BadRequest):
            self.admin_client.create_router(
                'name', flavor_id=self.flavor['id'], distributed=True)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import config

CONF = config.CONF


class QuotasTestBase(base.BaseAdminNetworkTest):

    required_extensions = ['quotas']

    @classmethod
    def resource_setup(cls):
        super(QuotasTestBase, cls).resource_setup()

    def _setup_quotas(self, project_id, **new_quotas):
        # Change quotas for tenant
        quota_set = self.admin_client.update_quotas(project_id,
                                                    **new_quotas)
        self.addCleanup(self._cleanup_quotas, project_id)
        return quota_set

    def _cleanup_quotas(self, project_id):
        # Try to clean up the resources. If it fails, then
        # assume that everything was already deleted, so
        # it is OK to continue.
        try:
            self.admin_client.reset_quotas(project_id)
        except lib_exc.NotFound:
            pass

    def _create_network(self, project_id):
        network = self.create_network(client=self.admin_client,
                                      tenant_id=project_id)
        self.addCleanup(self.admin_client.delete_network,
                        network['id'])
        return network

    def _create_port(self, **kwargs):
        port = self.admin_client.create_port(**kwargs)['port']
        self.addCleanup(self.admin_client.delete_port,
                        port['id'])
        return port


class QuotasTest(QuotasTestBase):
    """Test the Neutron API of Quotas.

    Tests the following operations in the Neutron API using the REST client for
    Neutron:

        list quotas for tenants who have non-default quota values
        show quotas for a specified tenant
        show detail quotas for a specified tenant
        update quotas for a specified tenant
        reset quotas to default values for a specified tenant

    v2.0 of the API is assumed.
    It is also assumed that the per-tenant quota extension API is configured
    in /etc/neutron/neutron.conf as follows:

        quota_driver = neutron.db.driver.DbQuotaDriver
    """

    @decorators.attr(type='gate')
    @decorators.idempotent_id('2390f766-836d-40ef-9aeb-e810d78207fb')
    def test_quotas(self):
        tenant_id = self.create_project()['id']
        new_quotas = {'network': 0, 'security_group': 0}

        # Change quotas for tenant
        quota_set = self._setup_quotas(tenant_id, **new_quotas)
        for key, value in new_quotas.items():
            self.assertEqual(value, quota_set[key])

        # Confirm our tenant is listed among tenants with non default quotas
        non_default_quotas = self.admin_client.list_quotas()
        found = False
        for qs in non_default_quotas['quotas']:
            if qs['tenant_id'] == tenant_id:
                self.assertEqual(tenant_id, qs['project_id'])
                found = True
        self.assertTrue(found)

        # Confirm from API quotas were changed as requested for tenant
        quota_set = self.admin_client.show_quotas(tenant_id)
        quota_set = quota_set['quota']
        for key, value in new_quotas.items():
            self.assertEqual(value, quota_set[key])

        # Reset quotas to default and confirm
        self.admin_client.reset_quotas(tenant_id)
        non_default_quotas = self.admin_client.list_quotas()
        for q in non_default_quotas['quotas']:
            self.assertNotEqual(tenant_id, q['tenant_id'])

    @decorators.idempotent_id('e974b5ba-090a-452c-a578-f9710151d9fc')
    @decorators.attr(type='gate')
    @utils.requires_ext(extension="quota_details", service="network")
    def test_detail_quotas(self):
        tenant_id = self.create_project()['id']
        new_quotas = {'network': {'used': 1, 'limit': 2, 'reserved': 0},
                      'port': {'used': 1, 'limit': 2, 'reserved': 0}}

        # create test resources
        network = self._create_network(tenant_id)
        post_body = {"network_id": network['id'],
                     "tenant_id": tenant_id}

        # NOTE(lucasagomes): Some backends such as OVN will create a port
        # to be used by the metadata agent upon creating a network. In
        # order to make this test more generic we need to calculate the
        # number of expected used ports after the network is created and
        # prior for the port being created
        ports = self.admin_client.list_ports(tenant_id=tenant_id)
        new_quotas['port']['used'] += len(ports['ports'])

        self._create_port(**post_body)

        # update quota limit for tenant
        new_quota = {'network': new_quotas['network']['limit'], 'port':
                     new_quotas['port']['limit']}
        quota_set = self._setup_quotas(tenant_id, **new_quota)

        # confirm from extended API quotas were changed
        # as requested for tenant
        quota_set = self.admin_client.show_details_quota(tenant_id)
        quota_set = quota_set['quota']
        for key, value in new_quotas.items():
            self.assertEqual(new_quotas[key]['limit'],
                             quota_set[key]['limit'])
            self.assertEqual(new_quotas[key]['reserved'],
                             quota_set[key]['reserved'])
            self.assertEqual(new_quotas[key]['used'],
                             quota_set[key]['used'])

        # validate 'default' action for old extension
        quota_limit = self.admin_client.show_quotas(tenant_id)['quota']
        for key, value in new_quotas.items():
            self.assertEqual(new_quotas[key]['limit'], quota_limit[key])



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools

from neutron_lib.api.definitions import portbindings as pb
from neutron_lib import constants as const
from neutron_lib.utils import test
from tempest.common import utils
from tempest.lib import decorators
import testtools

from neutron_tempest_plugin.common import utils as common_utils
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base
from neutron_tempest_plugin.scenario import test_dvr

CONF = config.CONF


class NetworkMigrationTestBase(base.BaseTempestTestCase,
                               test_dvr.NetworkTestMixin):
    credentials = ['primary', 'admin']
    force_tenant_isolation = False

    @classmethod
    @utils.requires_ext(extension="dvr", service="network")
    @utils.requires_ext(extension="l3-ha", service="network")
    def skip_checks(cls):
        super(NetworkMigrationTestBase, cls).skip_checks()

    def _check_update(self, router, is_dvr, is_ha):
        router = self.os_admin.network_client.show_router(router['id'])
        self.assertEqual(is_dvr, router['router']['distributed'])
        self.assertEqual(is_ha, router['router']['ha'])

    def _wait_until_port_deleted(self, router_id, device_owner):
        common_utils.wait_until_true(
            functools.partial(
                self._is_port_deleted,
                router_id,
                device_owner),
            timeout=300, sleep=5)

    def _is_port_deleted(self, router_id, device_owner):
        ports = self.os_admin.network_client.list_ports(
            device_id=router_id,
            device_owner=device_owner)
        return not ports.get('ports')

    def _wait_until_port_ready(self, router_id, device_owner):
        common_utils.wait_until_true(
            functools.partial(
                self._is_port_active,
                router_id,
                device_owner),
            timeout=300, sleep=5)

    def _wait_until_router_ports_down(self, router_id):

        def _is_port_down(port_id):
            port = self.os_admin.network_client.show_port(port_id).get('port')
            return port['status'] == const.DOWN

        ports = self.os_admin.network_client.list_ports(
            device_id=router_id).get('ports')
        for port in ports:
            common_utils.wait_until_true(
                functools.partial(_is_port_down, port['id']),
                timeout=300, sleep=5)

    def _is_port_active(self, router_id, device_owner):
        ports = self.os_admin.network_client.list_ports(
            device_id=router_id,
            device_owner=device_owner,
            status=const.ACTIVE).get('ports')
        if ports:
            if ports[0][pb.VIF_TYPE] not in [pb.VIF_TYPE_UNBOUND,
                                             pb.VIF_TYPE_BINDING_FAILED]:
                return True
        return False

    def _wait_until_router_ports_ready(self, router_id, dvr, ha):
        if dvr:
            self._wait_until_port_ready(
                router_id, const.DEVICE_OWNER_DVR_INTERFACE)
        if ha:
            self._wait_until_port_ready(
                router_id, const.DEVICE_OWNER_ROUTER_HA_INTF)
            if dvr:
                self._wait_until_port_ready(
                    router_id, const.DEVICE_OWNER_ROUTER_SNAT)
            else:
                self._wait_until_port_ready(
                    router_id, const.DEVICE_OWNER_HA_REPLICATED_INT)
        self._wait_until_port_ready(
            router_id, const.DEVICE_OWNER_ROUTER_GW)

    def _wait_until_router_ports_migrated(
            self, router_id, before_dvr, before_ha, after_dvr, after_ha):
        if before_ha and not after_ha:
            self._wait_until_port_deleted(
                router_id, const.DEVICE_OWNER_ROUTER_HA_INTF)
            self._wait_until_port_deleted(
                    router_id, const.DEVICE_OWNER_HA_REPLICATED_INT)
        if before_dvr and not after_dvr:
            self._wait_until_port_deleted(
                router_id, const.DEVICE_OWNER_DVR_INTERFACE)
            self._wait_until_port_deleted(
                router_id, const.DEVICE_OWNER_ROUTER_SNAT)
        self._wait_until_router_ports_ready(router_id, after_dvr, after_ha)

    def _test_migration(self, before_dvr, before_ha, after_dvr, after_ha):
        router = self.create_router_by_client(
            distributed=before_dvr, ha=before_ha,
            tenant_id=self.client.tenant_id, is_admin=True)

        self.setup_network_and_server(router=router)
        self._wait_until_router_ports_ready(
            router['id'], before_dvr, before_ha)
        self._check_connectivity()

        self.os_admin.network_client.update_router(
            router_id=router['id'], admin_state_up=False)
        self._wait_until_router_ports_down(router['id'])

        self.os_admin.network_client.update_router(
            router_id=router['id'], distributed=after_dvr, ha=after_ha)
        self._check_update(router, after_dvr, after_ha)

        self.os_admin.network_client.update_router(
            router_id=router['id'], admin_state_up=True)

        self._wait_until_router_ports_migrated(
            router['id'], before_dvr, before_ha, after_dvr, after_ha)
        self._check_connectivity()


class NetworkMigrationFromLegacy(NetworkMigrationTestBase):

    @decorators.idempotent_id('23724222-483a-4129-bc15-7a9278f3828b')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_legacy_to_dvr(self):
        self._test_migration(before_dvr=False, before_ha=False,
                             after_dvr=True, after_ha=False)

    @decorators.idempotent_id('09d85102-994f-4ff9-bf3e-17051145ca12')
    def test_from_legacy_to_ha(self):
        self._test_migration(before_dvr=False, before_ha=False,
                             after_dvr=False, after_ha=True)

    @decorators.idempotent_id('fe169f2c-6ed3-4eb0-8afe-2d540c4b49e2')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_legacy_to_dvr_ha(self):
        self._test_migration(before_dvr=False, before_ha=False,
                             after_dvr=True, after_ha=True)


class NetworkMigrationFromHA(NetworkMigrationTestBase):

    @decorators.idempotent_id('b4e68ac0-3b76-4306-ae8a-51cf4d363b22')
    def test_from_ha_to_legacy(self):
        self._test_migration(before_dvr=False, before_ha=True,
                             after_dvr=False, after_ha=False)

    @decorators.idempotent_id('42260eea-5d56-4d30-b62a-a62694dfe4d5')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_ha_to_dvr(self):
        self._test_migration(before_dvr=False, before_ha=True,
                             after_dvr=True, after_ha=False)

    @decorators.idempotent_id('e4149576-248b-43fa-9d0b-a5c2f51967ce')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_ha_to_dvr_ha(self):
        self._test_migration(before_dvr=False, before_ha=True,
                             after_dvr=True, after_ha=True)


class NetworkMigrationFromDVR(NetworkMigrationTestBase):

    @decorators.idempotent_id('e5cac02c-248d-4aac-bd5e-9d47c5197307')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_dvr_to_legacy(self):
        self._test_migration(before_dvr=True, before_ha=False,
                             after_dvr=False, after_ha=False)

    @decorators.idempotent_id('a00d5ad7-8509-4bb0-bdd2-7f1ee052d1cd')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_dvr_to_ha(self):
        self._test_migration(before_dvr=True, before_ha=False,
                             after_dvr=False, after_ha=True)

    @decorators.idempotent_id('25304a51-93a8-4cf3-9523-bce8b4eaecf8')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_dvr_to_dvr_ha(self):
        self._test_migration(before_dvr=True, before_ha=False,
                             after_dvr=True, after_ha=True)


class NetworkMigrationFromDVRHA(NetworkMigrationTestBase):

    @test.unstable_test("bug 1756301")
    @decorators.idempotent_id('1be9b2e2-379c-40a4-a269-6687b81df691')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_dvr_ha_to_legacy(self):
        self._test_migration(before_dvr=True, before_ha=True,
                             after_dvr=False, after_ha=False)

    @test.unstable_test("bug 1756301")
    @decorators.idempotent_id('55957267-4e84-4314-a2f7-7cd36a2df04b')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_dvr_ha_to_ha(self):
        self._test_migration(before_dvr=True, before_ha=True,
                             after_dvr=False, after_ha=True)

    @test.unstable_test("bug 1756301")
    @decorators.idempotent_id('d6bedff1-72be-4a9a-8ea2-dc037cd838e0')
    @testtools.skipUnless(
        CONF.neutron_plugin_options.l3_agent_mode == 'dvr_snat',
        "Need dvr_snat agent mode assumption.")
    def test_from_dvr_ha_to_dvr(self):
        self._test_migration(before_dvr=True, before_ha=True,
                             after_dvr=True, after_ha=False)



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from neutron_lib import constants
from tempest.common import compute
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from neutron_tempest_plugin.common import ip as ip_utils
from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base

CONF = config.CONF


class NetworkConnectivityTest(base.BaseTempestTestCase):
    credentials = ['primary', 'admin']

    @classmethod
    @utils.requires_ext(extension="router", service="network")
    def resource_setup(cls):
        super(NetworkConnectivityTest, cls).resource_setup()
        # Create keypair with admin privileges
        cls.keypair = cls.create_keypair()
        # Create security group with admin privileges
        cls.secgroup = cls.create_security_group(
            name=data_utils.rand_name('secgroup'))
        # Execute funcs to achieve ssh and ICMP capabilities
        cls.create_loginable_secgroup_rule(secgroup_id=cls.secgroup['id'])
        cls.create_pingable_secgroup_rule(secgroup_id=cls.secgroup['id'])

    def _create_servers(self, port_1, port_2):
        params = {
            'flavor_ref': CONF.compute.flavor_ref,
            'image_ref': CONF.compute.image_ref,
            'key_name': self.keypair['name']
        }
        vms = []
        vms.append(
            self.create_server(networks=[{'port': port_1['id']}], **params))

        if (CONF.compute.min_compute_nodes > 1 and
                compute.is_scheduler_filter_enabled("DifferentHostFilter")):
            params['scheduler_hints'] = {
                'different_host': [vms[0]['server']['id']]}

        vms.append(
            self.create_server(networks=[{'port': port_2['id']}], **params))

        for vm in vms:
            self.wait_for_server_active(vm['server'])
            self.wait_for_guest_os_ready(vm['server'])

        return vms

    @decorators.idempotent_id('8944b90d-1766-4669-bd8a-672b5d106bb7')
    def test_connectivity_through_2_routers(self):
        ap1_net = self.create_network()
        ap2_net = self.create_network()
        wan_net = self.create_network()
        ap1_subnet = self.create_subnet(
            ap1_net, cidr="10.10.210.0/24", gateway="10.10.210.254")
        ap2_subnet = self.create_subnet(
            ap2_net, cidr="10.10.220.0/24", gateway="10.10.220.254")
        self.create_subnet(
            wan_net, cidr="10.10.200.0/24", gateway="10.10.200.254")

        ap1_rt = self.create_router(
            router_name=data_utils.rand_name("ap1_rt"),
            admin_state_up=True,
            external_network_id=CONF.network.public_network_id)
        ap2_rt = self.create_router(
            router_name=data_utils.rand_name("ap2_rt"),
            admin_state_up=True)
        self._wait_for_router_ha_active(ap1_rt['id'])
        self._wait_for_router_ha_active(ap2_rt['id'])

        ap1_internal_port = self.create_port(
            ap1_net, security_groups=[self.secgroup['id']])
        ap2_internal_port = self.create_port(
            ap2_net, security_groups=[self.secgroup['id']])
        ap1_wan_port = self.create_port(wan_net)
        ap2_wan_port = self.create_port(wan_net)

        self.client.add_router_interface_with_port_id(
            ap1_rt['id'], ap1_wan_port['id'])
        self.client.add_router_interface_with_port_id(
            ap2_rt['id'], ap2_wan_port['id'])
        self.create_router_interface(ap1_rt['id'], ap1_subnet['id'])
        self.create_router_interface(ap2_rt['id'], ap2_subnet['id'])

        self.client.update_router(
            ap1_rt['id'],
            routes=[{"destination": ap2_subnet['cidr'],
                     "nexthop": ap2_wan_port['fixed_ips'][0]['ip_address']}])
        self.client.update_router(
            ap2_rt['id'],
            routes=[{"destination": ap1_subnet['cidr'],
                     "nexthop": ap1_wan_port['fixed_ips'][0]['ip_address']}])

        servers = self._create_servers(ap1_internal_port, ap2_internal_port)

        ap1_fip = self.create_and_associate_floatingip(
            ap1_internal_port['id'])
        ap1_sshclient = ssh.Client(
            ap1_fip['floating_ip_address'], CONF.validation.image_ssh_user,
            pkey=self.keypair['private_key'])

        self.check_remote_connectivity(
            ap1_sshclient, ap2_internal_port['fixed_ips'][0]['ip_address'],
            servers=servers)

    @decorators.idempotent_id('b72c3b77-3396-4144-b05d-9cd3c0099893')
    def test_connectivity_router_east_west_traffic(self):
        """This case is intended to test router east west taffic

        The case can be used in various scenarios: legacy/distributed router,
        same/different host.
        """
        net_1 = self.create_network()
        net_2 = self.create_network()
        subnet_1 = self.create_subnet(net_1, cidr="10.10.1.0/24")
        subnet_2 = self.create_subnet(net_2, cidr="10.10.2.0/24")

        router = self.create_router(
            router_name=data_utils.rand_name("east_west_traffic_router"),
            admin_state_up=True,
            external_network_id=CONF.network.public_network_id)
        self._wait_for_router_ha_active(router['id'])

        internal_port_1 = self.create_port(
            net_1, security_groups=[self.secgroup['id']])
        internal_port_2 = self.create_port(
            net_2, security_groups=[self.secgroup['id']])

        self.create_router_interface(router['id'], subnet_1['id'])
        self.create_router_interface(router['id'], subnet_2['id'])

        servers = self._create_servers(internal_port_1, internal_port_2)

        fip = self.create_and_associate_floatingip(
            internal_port_1['id'])
        sshclient = ssh.Client(
            fip['floating_ip_address'], CONF.validation.image_ssh_user,
            pkey=self.keypair['private_key'])

        self.check_remote_connectivity(
            sshclient, internal_port_2['fixed_ips'][0]['ip_address'],
            ping_count=10, servers=servers)

    @utils.requires_ext(extension="dvr", service="network")
    @decorators.idempotent_id('69d3650a-5c32-40bc-ae56-5c4c849ddd37')
    def test_connectivity_dvr_and_no_dvr_routers_in_same_subnet(self):
        """This test case tests connectivity between vm and 2 routers.

        Subnet is connected to dvr and non-dvr routers in the same time, test
        ensures that connectivity from VM to both routers is working.

        Test scenario: (NOTE: 10.1.0.0/24 private CIDR is used as an example)
        +----------------+                  +------------+
        | Non-dvr router |                  | DVR router |
        |                |                  |            |
        |    10.1.0.1    |                  |  10.1.0.x  |
        +-------+--------+                  +-----+------+
                |                                 |
                |         10.1.0.0/24             |
                +----------------+----------------+
                                 |
                               +-+-+
                               |VM |
                               +---+

        where:
        10.1.0.1 - is subnet's gateway IP address,
        10.1.0.x - is any other IP address taken from subnet's range

        Test ensures that both 10.1.0.1 and 10.1.0.x IP addresses are
        reachable from VM.
        """
        ext_network = self.client.show_network(self.external_network_id)
        for ext_subnetid in ext_network['network']['subnets']:
            ext_subnet = self.os_admin.network_client.show_subnet(ext_subnetid)
            ext_cidr = ext_subnet['subnet']['cidr']
            if ext_subnet['subnet']['ip_version'] == constants.IP_VERSION_4:
                break
        else:
            self.fail('No IPv4 subnet was found in external network %s' %
                      ext_network['network']['id'])

        subnet_cidr = ip_utils.find_valid_cidr(used_cidr=ext_cidr)
        gw_ip = netaddr.IPAddress(subnet_cidr.first + 1)

        network = self.create_network()
        subnet = self.create_subnet(
            network, cidr=str(subnet_cidr), gateway=str(gw_ip))

        non_dvr_router = self.create_router_by_client(
            tenant_id=self.client.tenant_id,
            is_admin=True,
            router_name=data_utils.rand_name("nondvr-2-routers-same-network"),
            admin_state_up=True,
            distributed=False)
        self.create_router_interface(non_dvr_router['id'], subnet['id'])

        dvr_router = self.create_router_by_client(
            tenant_id=self.client.tenant_id,
            is_admin=True,
            router_name=data_utils.rand_name("dvr-2-rotuers-same-network"),
            admin_state_up=True,
            distributed=True)
        dvr_router_port = self.create_port(network)
        self.client.add_router_interface_with_port_id(
            dvr_router['id'], dvr_router_port['id'])

        vm = self.create_server(
            flavor_ref=CONF.compute.flavor_ref,
            image_ref=CONF.compute.image_ref,
            key_name=self.keypair['name'],
            networks=[{'uuid': network['id']}],
            security_groups=[{'name': self.secgroup['name']}])
        self.wait_for_server_active(vm['server'])
        self.wait_for_guest_os_ready(vm['server'])

        vm_port = self.client.list_ports(
            network_id=network['id'], device_id=vm['server']['id'])['ports'][0]
        fip = self.create_and_associate_floatingip(vm_port['id'])

        sshclient = ssh.Client(
            fip['floating_ip_address'], CONF.validation.image_ssh_user,
            pkey=self.keypair['private_key'])

        self.check_remote_connectivity(
            sshclient, str(gw_ip), ping_count=10, servers=[vm])
        self.check_remote_connectivity(
            sshclient, dvr_router_port['fixed_ips'][0]['ip_address'],
            ping_count=10, servers=[vm])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import tempest.api.network.base as test
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils

from neutron_tempest_plugin.tap_as_a_service.services import taas_client

CONF = config.CONF


class BaseTaasTest(test.BaseAdminNetworkTest):

    @classmethod
    def resource_setup(cls):
        super(BaseTaasTest, cls).resource_setup()
        os_primary = cls.os_primary
        cls.tap_services_client = taas_client.TapServicesClient(
            os_primary.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **os_primary.default_params)
        cls.tap_flows_client = taas_client.TapFlowsClient(
            os_primary.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **os_primary.default_params)

    def create_tap_service(self, **kwargs):
        body = self.tap_services_client.create_tap_service(
            name=data_utils.rand_name("tap_service"),
            **kwargs)
        tap_service = body['tap_service']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.tap_services_client.delete_tap_service,
                        tap_service['id'])
        return tap_service

    def create_tap_flow(self, **kwargs):
        body = self.tap_flows_client.create_tap_flow(
            name=data_utils.rand_name("tap_service"),
            **kwargs)
        tap_flow = body['tap_flow']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.tap_flows_client.delete_tap_flow,
                        tap_flow['id'])
        return tap_flow

    def update_tap_service(self, tap_service_id, **kwargs):
        body = self.tap_services_client.update_tap_service(
            tap_service_id,
            **kwargs)
        tap_service = body['tap_service']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.tap_services_client.delete_tap_service,
                        tap_service['id'])

    def update_tap_flow(self, tap_flow_id, **kwargs):
        body = self.tap_flows_client.update_tap_flow(
            tap_flow_id,
            **kwargs)
        tap_flow = body['tap_flow']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.tap_flows_client.delete_tap_flow,
                        tap_flow['id'])



# Copyright 2018 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from neutron_tempest_plugin.api import base


class ListAvailableZonesTest(base.BaseNetworkTest):

    @decorators.idempotent_id('5a8a8a1a-c265-11e8-a611-080027758b73')
    @utils.requires_ext(extension="availability_zone",
                        service="network")
    def test_list_available_zones(self):
        body = self.client.list_availability_zones()
        self.assertIsNotNone(body)
        self.assertIsInstance(body['availability_zones'], list)



# Copyright 2019 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants
from oslo_log import log
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.common import ssh
from neutron_tempest_plugin.common import utils
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base

CONF = config.CONF

LOG = log.getLogger(__name__)


class PortForwardingTestJSON(base.BaseTempestTestCase):

    credentials = ['primary', 'admin']
    required_extensions = ['router', 'floating-ip-port-forwarding']

    @classmethod
    def resource_setup(cls):
        super(PortForwardingTestJSON, cls).resource_setup()
        cls.skip_if_no_extension_enabled_in_l3_agents("port_forwarding")
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.router = cls.create_router_by_client()
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.fip = cls.create_floatingip()
        cls.secgroup = cls.create_security_group(
            name=data_utils.rand_name("test_port_secgroup"))
        cls.create_loginable_secgroup_rule(secgroup_id=cls.secgroup['id'])
        udp_sg_rule = {'protocol': constants.PROTO_NAME_UDP,
                       'direction': constants.INGRESS_DIRECTION,
                       'remote_ip_prefix': '0.0.0.0/0'}
        cls.create_secgroup_rules(
            [udp_sg_rule], secgroup_id=cls.secgroup['id'])
        cls.keypair = cls.create_keypair()

    def _prepare_resources(self, num_servers, internal_tcp_port=22,
                           external_port_base=1025):
        servers = []
        for i in range(1, num_servers + 1):
            internal_udp_port = internal_tcp_port + 10
            external_tcp_port = external_port_base + i
            external_udp_port = external_tcp_port + 10
            name = data_utils.rand_name("server-%s" % i)
            port = self.create_port(
                self.network,
                security_groups=[self.secgroup['id']])
            server = self.create_server(
                flavor_ref=CONF.compute.flavor_ref,
                image_ref=CONF.compute.image_ref,
                key_name=self.keypair['name'], name=name,
                networks=[{'port': port['id']}])['server']
            server['name'] = name
            self.wait_for_server_active(server)
            self.wait_for_guest_os_ready(server)
            server['port_forwarding_tcp'] = self.create_port_forwarding(
                self.fip['id'],
                internal_port_id=port['id'],
                internal_ip_address=port['fixed_ips'][0]['ip_address'],
                internal_port=internal_tcp_port,
                external_port=external_tcp_port,
                protocol=constants.PROTO_NAME_TCP)
            server['port_forwarding_udp'] = self.create_port_forwarding(
                self.fip['id'],
                internal_port_id=port['id'],
                internal_ip_address=port['fixed_ips'][0]['ip_address'],
                internal_port=internal_udp_port,
                external_port=external_udp_port,
                protocol=constants.PROTO_NAME_UDP)
            servers.append(server)
        return servers

    def _test_udp_port_forwarding(self, servers, timeout=None):

        def _message_received(server, ssh_client, expected_msg):
            self.nc_listen(ssh_client,
                           server['port_forwarding_udp']['internal_port'],
                           constants.PROTO_NAME_UDP,
                           expected_msg,
                           [server])
            received_msg = self.nc_client(
                self.fip['floating_ip_address'],
                server['port_forwarding_udp']['external_port'],
                constants.PROTO_NAME_UDP)
            return expected_msg in received_msg

        for server in servers:
            expected_msg = "%s-UDP-test" % server['name']
            ssh_client = ssh.Client(
                self.fip['floating_ip_address'],
                CONF.validation.image_ssh_user,
                pkey=self.keypair['private_key'],
                port=server['port_forwarding_tcp']['external_port'])
            wait_params = {
                'exception': RuntimeError(
                    "Timed out waiting for message from server {!r} ".format(
                        server['id']))
            }
            if timeout:
                wait_params['timeout'] = timeout
            utils.wait_until_true(
                lambda: _message_received(server, ssh_client, expected_msg),
                **wait_params)

    @decorators.idempotent_id('ab40fc48-ca8d-41a0-b2a3-f6679c847bfe')
    def test_port_forwarding_to_2_servers(self):
        servers = self._prepare_resources(num_servers=2,
                                          external_port_base=1035)
        # Test TCP port forwarding by SSH to each server
        self.check_servers_hostnames(servers)
        # And now test UDP port forwarding using nc
        self._test_udp_port_forwarding(servers)

    @decorators.idempotent_id('aa19d46c-a4a6-11ea-bb37-0242ac130002')
    def test_port_forwarding_editing_and_deleting_tcp_rule(self):
        test_ext_port = 3333
        server = self._prepare_resources(num_servers=1,
                                         external_port_base=1045)
        fip_id = server[0]['port_forwarding_tcp']['floatingip_id']
        pf_id = server[0]['port_forwarding_tcp']['id']

        # Check connectivity with the original parameters
        self.check_servers_hostnames(server)

        def fip_pf_connectivity(test_ssh_connect_timeout=60):
            try:
                self.check_servers_hostnames(
                    server, timeout=test_ssh_connect_timeout)
                return True
            except (AssertionError, lib_exc.SSHTimeout):
                return False

        def no_fip_pf_connectivity():
            return not fip_pf_connectivity(6)

        # Update external port and check connectivity with original parameters
        # Port under server[0]['port_forwarding_tcp']['external_port'] should
        # not answer at this point.
        self.client.update_port_forwarding(fip_id, pf_id,
                                           external_port=test_ext_port)
        utils.wait_until_true(
            no_fip_pf_connectivity,
            exception=RuntimeError(
                "Connection to the server {!r} through "
                "port {!r} is still possible.".format(
                    server[0]['id'],
                    server[0]['port_forwarding_tcp']['external_port'])))

        # Check connectivity with the new parameters
        server[0]['port_forwarding_tcp']['external_port'] = test_ext_port
        utils.wait_until_true(
            fip_pf_connectivity,
            exception=RuntimeError(
                "Connection to the server {!r} through "
                "port {!r} is not possible.".format(
                    server[0]['id'],
                    server[0]['port_forwarding_tcp']['external_port'])))

        # Remove port forwarding and ensure connection stops working.
        self.client.delete_port_forwarding(fip_id, pf_id)
        self.assertRaises(lib_exc.NotFound, self.client.get_port_forwarding,
                          fip_id, pf_id)
        utils.wait_until_true(
            no_fip_pf_connectivity,
            exception=RuntimeError(
                "Connection to the server {!r} through "
                "port {!r} is still possible.".format(
                    server[0]['id'],
                    server[0]['port_forwarding_tcp']['external_port'])))

    @decorators.idempotent_id('6d05b1b2-6109-4c30-b402-1503f4634acb')
    def test_port_forwarding_editing_and_deleting_udp_rule(self):
        test_ext_port = 3344
        server = self._prepare_resources(num_servers=1,
                                         external_port_base=1055)
        fip_id = server[0]['port_forwarding_udp']['floatingip_id']
        pf_id = server[0]['port_forwarding_udp']['id']

        # Check connectivity with the original parameters
        self.check_servers_hostnames(server)

        def fip_pf_udp_connectivity(test_udp_timeout=60):
            try:
                self._test_udp_port_forwarding(server, test_udp_timeout)
                return True
            except (AssertionError, RuntimeError):
                return False

        def no_fip_pf_udp_connectivity():
            return not fip_pf_udp_connectivity(6)

        # Update external port and check connectivity with original parameters
        # Port under server[0]['port_forwarding_udp']['external_port'] should
        # not answer at this point.
        self.client.update_port_forwarding(fip_id, pf_id,
                                           external_port=test_ext_port)
        utils.wait_until_true(
            no_fip_pf_udp_connectivity,
            exception=RuntimeError(
                "Connection to the server {!r} through "
                "port {!r} is still possible.".format(
                    server[0]['id'],
                    server[0]['port_forwarding_udp']['external_port'])))

        # Check connectivity with the new parameters
        server[0]['port_forwarding_udp']['external_port'] = test_ext_port
        utils.wait_until_true(
            fip_pf_udp_connectivity,
            exception=RuntimeError(
                "Connection to the server {!r} through "
                "port {!r} is not possible.".format(
                    server[0]['id'],
                    server[0]['port_forwarding_udp']['external_port'])))

        # Remove port forwarding and ensure connection stops working.
        self.client.delete_port_forwarding(fip_id, pf_id)
        self.assertRaises(lib_exc.NotFound, self.client.get_port_forwarding,
                          fip_id, pf_id)
        utils.wait_until_true(
            no_fip_pf_udp_connectivity,
            exception=RuntimeError(
                "Connection to the server {!r} through "
                "port {!r} is still possible.".format(
                    server[0]['id'],
                    server[0]['port_forwarding_udp']['external_port'])))

    @decorators.idempotent_id('5971881d-06a0-459e-b636-ce5d1929e2d4')
    def test_port_forwarding_to_2_fixed_ips(self):
        port = self.create_port(self.network,
            security_groups=[self.secgroup['id']])
        name = data_utils.rand_name("server-0")
        server = self.create_server(flavor_ref=CONF.compute.flavor_ref,
            image_ref=CONF.compute.image_ref, key_name=self.keypair['name'],
            name=name, networks=[{'port': port['id']}])['server']
        server['name'] = name
        self.wait_for_server_active(server)
        self.wait_for_guest_os_ready(server)

        # Add a second fixed_ip address to port (same subnet)
        internal_subnet_id = port['fixed_ips'][0]['subnet_id']
        port['fixed_ips'].append({'subnet_id': internal_subnet_id})
        port = self.update_port(port, fixed_ips=port['fixed_ips'])
        internal_ip_address1 = port['fixed_ips'][0]['ip_address']
        internal_ip_address2 = port['fixed_ips'][1]['ip_address']
        pfs = []
        for ip_address, external_port in [(internal_ip_address1, 1066),
                                          (internal_ip_address2, 1067)]:
            pf = self.create_port_forwarding(
                self.fip['id'], internal_port_id=port['id'],
                internal_ip_address=ip_address,
                internal_port=22, external_port=external_port,
                protocol=constants.PROTO_NAME_TCP)
            pfs.append(pf)

        test_ssh_connect_timeout = 32
        number_of_connects = 0
        for pf in pfs:
            try:
                self.check_servers_hostnames(
                    [server], timeout=test_ssh_connect_timeout,
                    external_port=pf['external_port'])
                number_of_connects += 1
            except (AssertionError, lib_exc.SSHTimeout):
                pass

        # TODO(flaviof): Quite possibly, the server is using only one of the
        # fixed ips associated with the neutron port. Being so, we should not
        # fail the test, as long as at least one connection was successful.
        self.assertGreaterEqual(
            number_of_connects, 1, "Did not connect via FIP port forwarding")



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import locale
import os
import socket
import time

from oslo_log import log
import paramiko
from tempest.lib.common import ssh
from tempest.lib import exceptions
import tenacity

from neutron_tempest_plugin import config
from neutron_tempest_plugin import exceptions as exc


CONF = config.CONF
LOG = log.getLogger(__name__)


RETRY_EXCEPTIONS = (exceptions.TimeoutException, paramiko.SSHException,
                    socket.error, TimeoutError)


class Client(ssh.Client):

    default_ssh_lang = 'en_US.UTF-8'

    timeout = CONF.validation.ssh_timeout

    proxy_jump_host = CONF.neutron_plugin_options.ssh_proxy_jump_host
    proxy_jump_username = CONF.neutron_plugin_options.ssh_proxy_jump_username
    proxy_jump_password = CONF.neutron_plugin_options.ssh_proxy_jump_password
    proxy_jump_keyfile = CONF.neutron_plugin_options.ssh_proxy_jump_keyfile
    proxy_jump_port = CONF.neutron_plugin_options.ssh_proxy_jump_port

    def __init__(self, host, username, password=None, timeout=None, pkey=None,
                 channel_timeout=10, look_for_keys=False, key_filename=None,
                 port=22, proxy_client=None, create_proxy_client=True):

        timeout = timeout or self.timeout

        if not proxy_client and create_proxy_client and self.proxy_jump_host:
            # Perform all SSH connections passing through configured SSH server
            proxy_client = self.create_proxy_client(
                timeout=timeout, channel_timeout=channel_timeout)

        super(Client, self).__init__(
            host=host, username=username, password=password, timeout=timeout,
            pkey=pkey, channel_timeout=channel_timeout,
            look_for_keys=look_for_keys, key_filename=key_filename, port=port,
            proxy_client=proxy_client,
            ssh_key_type=CONF.validation.ssh_key_type)

    @classmethod
    def create_proxy_client(cls, look_for_keys=True, **kwargs):
        host = cls.proxy_jump_host
        if not host:
            # proxy_jump_host string cannot be empty or None
            raise ValueError(
                "'proxy_jump_host' configuration option is empty.")

        # Let accept an empty string as a synonymous of default value on below
        # options
        password = cls.proxy_jump_password or None
        key_file = cls.proxy_jump_keyfile or None
        username = cls.proxy_jump_username

        # Port must be a positive integer
        port = cls.proxy_jump_port
        if port <= 0 or port > 65535:
            raise ValueError(
                "Invalid value for 'proxy_jump_port' configuration option: "
                "{!r}".format(port))

        login = "{username}@{host}:{port}".format(username=username, host=host,
                                                  port=port)

        if key_file:
            # expand ~ character with user HOME directory
            key_file = os.path.expanduser(key_file)
            if os.path.isfile(key_file):
                LOG.debug("Going to create SSH connection to %r using key "
                          "file: %s", login, key_file)

            else:
                # This message could help the user to identify a
                # mis-configuration in tempest.conf
                raise ValueError(
                    "Cannot find file specified as 'proxy_jump_keyfile' "
                    "option: {!r}".format(key_file))

        elif password:
            LOG.debug("Going to create SSH connection to %r using password.",
                      login)

        elif look_for_keys:
            # This message could help the user to identify a mis-configuration
            # in tempest.conf
            LOG.info("Both 'proxy_jump_password' and 'proxy_jump_keyfile' "
                     "options are empty. Going to create SSH connection to %r "
                     "looking for key file location into %r directory.",
                     login, os.path.expanduser('~/.ssh'))
        else:
            # An user that forces look_for_keys=False should really know what
            # he really wants
            LOG.warning("No authentication method provided to create an SSH "
                        "connection to %r. If it fails, then please "
                        "set 'proxy_jump_keyfile' to provide a valid SSH key "
                        "file.", login)

        return Client(
            host=host, username=username, password=password,
            look_for_keys=look_for_keys, key_filename=key_file,
            port=port, create_proxy_client=False, **kwargs)

    def connect(self, *args, **kwargs):
        """Creates paramiko.SSHClient and connect it to remote SSH server

        :returns: paramiko.Client connected to remote server.

        :raises tempest.lib.exceptions.SSHTimeout: in case it fails to connect
        to remote server.
        """
        return super(Client, self)._get_ssh_connection(*args, **kwargs)

    # This overrides superclass test_connection_auth method forbidding it to
    # close connection
    test_connection_auth = connect

    def open_session(self):
        """Gets connection to SSH server and open a new paramiko.Channel

        :returns: new paramiko.Channel
        """

        client = self.connect()

        try:
            return client.get_transport().open_session()
        except paramiko.SSHException:
            # the request is rejected, the session ends prematurely or
            # there is a timeout opening a channel
            LOG.exception("Unable to open SSH session")
            raise exceptions.SSHTimeout(host=self.host,
                                        user=self.username,
                                        password=self.password)

    @tenacity.retry(
        stop=tenacity.stop_after_attempt(10),
        wait=tenacity.wait_fixed(1),
        retry=tenacity.retry_if_exception_type(RETRY_EXCEPTIONS),
        reraise=True)
    def exec_command(self, cmd, encoding="utf-8", timeout=None):
        if timeout:
            original_timeout = self.timeout
            self.timeout = timeout
        try:
            return super(Client, self).exec_command(cmd=cmd, encoding=encoding)
        finally:
            if timeout:
                self.timeout = original_timeout

    def execute_script(self, script, become_root=False, combine_stderr=False,
                       shell='sh -eux', timeout=None, **params):
        """Connect to remote machine and executes script.

        Implementation note: it passes script lines to shell interpreter via
        STDIN. Therefore script line number could be not available to some
        script interpreters for debugging porposes.

        :param script: script lines to be executed.

        :param become_root: executes interpreter as root with sudo.

        :param combine_stderr (bool): whenever to redirect STDERR to STDOUT so
        that output from both streams are returned together. True by default.

        :param shell: command line used to launch script interpreter. By
        default it executes Bash with -eux options enabled. This means that
        any command returning non-zero exist status or any any undefined
        variable would interrupt script execution with an error and every
        command executed by the script is going to be traced to STDERR.

        :param timeout: time in seconds to wait before brutally aborting
        script execution.

        :param **params: script parameter values to be assigned at the
        beginning of the script.

        :returns output written by script to STDOUT.

        :raises tempest.lib.exceptions.SSHTimeout: in case it fails to connect
        to remote server or it fails to open a channel.

        :raises tempest.lib.exceptions.SSHExecCommandFailed: in case command
        script exits with non zero exit status or times out.
        """

        if params:
            # Append script parameters at the beginning of the script
            header = ''.join(sorted(["{!s}={!s}\n".format(k, v)
                                     for k, v in params.items()]))
            script = header + '\n' + script

        timeout = timeout or self.timeout
        end_of_time = time.time() + timeout
        output_data = b''
        error_data = b''
        exit_status = None

        channel = self.open_session()
        with channel:

            # Combine STOUT and STDERR to have to handle with only one stream
            channel.set_combine_stderr(combine_stderr)

            # Update local environment
            lang, encoding = locale.getlocale()
            if not lang:
                lang, encoding = locale.getdefaultlocale()
            _locale = '.'.join([lang, encoding])
            channel.update_environment({'LC_ALL': _locale,
                                        'LANG': _locale})

            if become_root:
                shell = 'sudo ' + shell
            # Spawn a Bash
            channel.exec_command(shell)

            end_of_script = False
            lines_iterator = iter(script.splitlines())
            while (not channel.exit_status_ready() and
                   time.time() < end_of_time):
                # Drain incoming data buffers
                while channel.recv_ready():
                    output_data += channel.recv(self.buf_size)
                while channel.recv_stderr_ready():
                    error_data += channel.recv_stderr(self.buf_size)

                if not end_of_script and channel.send_ready():
                    try:
                        line = next(lines_iterator)
                    except StopIteration:
                        # Finalize Bash script execution
                        channel.shutdown_write()
                        end_of_script = True
                    else:
                        # Send script to Bash STDIN line by line
                        channel.send((line + '\n').encode(encoding))
                        continue

                time.sleep(.1)

            # Get exit status and drain incoming data buffers
            if channel.exit_status_ready():
                exit_status = channel.recv_exit_status()
            while channel.recv_ready():
                output_data += channel.recv(self.buf_size)
            while channel.recv_stderr_ready():
                error_data += channel.recv_stderr(self.buf_size)

        stdout = _buffer_to_string(output_data, encoding)
        if exit_status == 0:
            return stdout

        stderr = _buffer_to_string(error_data, encoding)
        if exit_status is None:
            raise exc.SSHScriptTimeoutExpired(
                command=shell, host=self.host, script=script, stderr=stderr,
                stdout=stdout, timeout=timeout)
        else:
            raise exc.SSHScriptFailed(
                command=shell, host=self.host, script=script, stderr=stderr,
                stdout=stdout, exit_status=exit_status)

    def get_hostname(self):
        """Retrieve the remote machine hostname"""
        try:
            return self.exec_command('hostname')
        except exceptions.SSHExecCommandFailed:
            return self.exec_command('cat /etc/hostname')


def _buffer_to_string(data_buffer, encoding):
    return data_buffer.decode(encoding).replace("\r\n", "\n").replace(
        "\r", "\n")



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants
from neutron_lib.db import constants as db_const
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin.api import base_security_groups
from neutron_tempest_plugin.api import test_security_groups


LONG_NAME_NG = 'x' * (db_const.NAME_FIELD_SIZE + 1)


class NegativeSecGroupTest(base.BaseNetworkTest):

    required_extensions = ['security-group']

    @classmethod
    def resource_setup(cls):
        super(NegativeSecGroupTest, cls).resource_setup()
        cls.network = cls.create_network()

    @decorators.attr(type='negative')
    @decorators.idempotent_id('594edfa8-9a5b-438e-9344-49aece337d49')
    def test_create_security_group_with_too_long_name(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.create_security_group,
                          name=LONG_NAME_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('b6b79838-7430-4d3f-8e07-51dfb61802c2')
    def test_create_security_group_with_boolean_type_name(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.create_security_group,
                          name=True)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('55100aa8-b24f-333c-0bef-64eefd85f15c')
    def test_update_default_security_group_name(self):
        security_group = self.client.list_security_groups(name='default')[
            'security_groups'][0]
        self.assertRaises(lib_exc.Conflict, self.client.update_security_group,
                          security_group['id'], name='test')

    @decorators.attr(type='negative')
    @decorators.idempotent_id('c8510dd8-c3a8-4df9-ae44-24354db50960')
    def test_update_security_group_with_too_long_name(self):
        security_group = self.client.list_security_groups(name='default')[
            'security_groups'][0]
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_security_group,
                          security_group['id'], name=LONG_NAME_NG)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('d9a14917-f66f-4eca-ab72-018563917f1b')
    def test_update_security_group_with_boolean_type_name(self):
        security_group = self.client.list_security_groups(name='default')[
            'security_groups'][0]
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_security_group,
                          security_group['id'], name=True)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('3200b1a8-d73b-48e9-b03f-e891a4abe2d3')
    def test_delete_in_use_sec_group(self):
        security_group = self.create_security_group()
        self.create_port(network=self.network,
                         security_groups=[security_group['id']])
        self.assertRaises(lib_exc.Conflict,
                          self.os_primary.network_client.delete_security_group,
                          security_group_id=security_group['id'])

    @decorators.attr(type='negative')
    @decorators.idempotent_id('867d67c3-7e26-4288-a27b-e3d0649ee54b')
    def test_assign_sec_group_twice(self):
        net = self.create_network()
        port = self.create_port(net)
        sg = self.create_security_group()
        self.assertRaises(lib_exc.BadRequest,
                          self.update_port,
                          port,
                          **{'security_groups': [sg['id'], sg['id']]})

    @decorators.attr(type='negative')
    @decorators.idempotent_id('d5ecb408-eb7e-47c1-a56f-353967dbd1c2')
    def test_assign_nonexistent_sec_group(self):
        net = self.create_network()
        port = self.create_port(net)
        self.assertRaises(lib_exc.NotFound,
                          self.update_port,
                          port,
                          **{'security_groups': [data_utils.rand_uuid()]})

    @decorators.attr(type='negative')
    @decorators.idempotent_id('98ef378d-81a2-43f6-bb6f-735c04cdef91')
    def test_no_sec_group_changes_after_assignment_failure(self):
        net = self.create_network()
        port = self.create_port(net)
        sg_list_before_failure = port['security_groups']
        self.assertRaises(lib_exc.NotFound,
                          self.update_port,
                          port,
                          **{'security_groups': [data_utils.rand_uuid()]})
        port_details_new = self.client.show_port(port['id'])['port']
        sg_list_after_failure = port_details_new['security_groups']
        self.assertEqual(sg_list_before_failure, sg_list_after_failure)


class NegativeSecGroupIPv6Test(NegativeSecGroupTest):
    _ip_version = constants.IP_VERSION_6


class NegativeSecGroupProtocolTest(base.BaseNetworkTest):

    def _test_create_security_group_rule_with_bad_protocols(self, protocols):
        security_group = self.create_security_group()

        # bad protocols can include v6 protocols because self.ethertype is v4
        for protocol in protocols:
            self.assertRaises(
                lib_exc.BadRequest,
                self.client.create_security_group_rule,
                security_group_id=security_group['id'],
                protocol=protocol, direction=constants.INGRESS_DIRECTION,
                ethertype=self.ethertype)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('cccbb0f3-c273-43ed-b3fc-1efc48833810')
    def test_create_security_group_rule_with_ipv6_protocol_names(self):
        self._test_create_security_group_rule_with_bad_protocols(
            base_security_groups.V6_PROTOCOL_NAMES)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8aa636bd-7060-4fdf-b722-cdae28e2f1ef')
    def test_create_security_group_rule_with_ipv6_protocol_integers(self):
        self._test_create_security_group_rule_with_bad_protocols(
            base_security_groups.V6_PROTOCOL_INTS)


class NegativeSecGroupQuotaTest(test_security_groups.BaseSecGroupQuota):

    credentials = ['primary', 'admin']
    required_extensions = ['security-group', 'quotas']

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('63f00cba-fcf5-4000-a3ee-eca58a1795c1')
    def test_create_excess_sg(self):
        self._set_sg_quota(0)
        self.assertRaises(lib_exc.Conflict, self.create_security_group)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('90a83445-bbc2-49d8-8c85-a111c08cd7fb')
    def test_sg_quota_incorrect_values(self):
        values = [-2, 2147483648, "value"]
        for value in values:
            self.assertRaises(lib_exc.BadRequest, self._set_sg_quota, value)


class NegativeSecGroupRulesQuotaTest(
        test_security_groups.BaseSecGroupRulesQuota):

    credentials = ['primary', 'admin']
    required_extensions = ['security-group', 'quotas']

    def setUp(self):
        super(NegativeSecGroupRulesQuotaTest, self).setUp()
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_client.reset_quotas, self.client.tenant_id)
        self._set_sg_rules_quota(10)

    @decorators.idempotent_id('8336e6ea-2e0a-4a1a-8673-a6f81b577d57')
    def test_sg_creation_with_insufficient_sg_rules_quota(self):
        self._set_sg_rules_quota(0)
        self.assertRaises(lib_exc.Conflict, self.create_security_group)


class NegativeStatelessSecGroupTest(base.BaseNetworkTest):

    required_extensions = ['security-group', 'stateful-security-group']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.network = cls.create_network()
        cls.stateless_sg = cls.create_security_group(stateful=False)
        cls.stateful_sg = cls.create_security_group(stateful=True)

    @decorators.idempotent_id('9e85ce0d-37b2-4044-88a8-09ae965069ba')
    def test_create_port_with_stateful_and_stateless_sg(self):
        self.assertRaises(
            lib_exc.Conflict,
            self.create_port,
            network=self.network,
            security_groups=[self.stateful_sg['id'], self.stateless_sg['id']])

    def _test_adding_sg_to_port_with_different_type_of_sg(
            self, initial_sg, updated_sg):
        port = self.create_port(
            network=self.network,
            security_groups=[initial_sg['id']]
        )
        self.assertRaises(
            lib_exc.Conflict,
            self.update_port,
            port,
            security_groups=[initial_sg['id'], updated_sg['id']]
        )

    @decorators.idempotent_id('63374580-3154-410b-ab31-e98a136094f8')
    def test_adding_stateful_sg_to_port_with_stateless_sg(self):
        self._test_adding_sg_to_port_with_different_type_of_sg(
            self.stateless_sg, self.stateful_sg)

    @decorators.idempotent_id('3854a4c6-4ace-4133-be83-4a2820ede06f')
    def test_adding_stateless_sg_to_port_with_stateful_sg(self):
        self._test_adding_sg_to_port_with_different_type_of_sg(
            self.stateful_sg, self.stateless_sg)

    def _test_update_used_sg(self, security_group):
        self.create_port(
            network=self.network,
            security_groups=[security_group['id']]
        )
        self.assertRaises(
            lib_exc.Conflict,
            self.client.update_security_group,
            security_group['id'],
            stateful=not security_group['stateful']
        )

    @decorators.idempotent_id('5e1e3053-16dc-4f0b-a327-ff953f527248')
    def test_update_used_stateless_sg_to_stateful(self):
        self._test_update_used_sg(self.stateless_sg)

    @decorators.idempotent_id('afe4d777-7a98-44ed-a1dc-588861f6daba')
    def test_update_used_stateful_sg_to_stateless(self):
        self._test_update_used_sg(self.stateful_sg)



# Copyright (c) 2015 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from neutron_lib import constants as nl_constants
from tempest import config
from tempest import exceptions
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.fwaas.services import v2_client


CONF = config.CONF


class FWaaSClientMixin(object):

    @classmethod
    def resource_setup(cls):
        super(FWaaSClientMixin, cls).resource_setup()
        manager = cls.os_primary
        cls.firewall_groups_client = v2_client.FirewallGroupsClient(
            manager.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **manager.default_params)
        cls.firewall_policies_client = v2_client.FirewallPoliciesClient(
            manager.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **manager.default_params)
        cls.firewall_rules_client = v2_client.FirewallRulesClient(
            manager.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **manager.default_params)

    def create_firewall_rule(self, **kwargs):
        body = self.firewall_rules_client.create_firewall_rule(
            name=data_utils.rand_name("fw-rule"),
            **kwargs)
        fw_rule = body['firewall_rule']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.firewall_rules_client.delete_firewall_rule,
                        fw_rule['id'])
        return fw_rule

    def create_firewall_policy(self, **kwargs):
        body = self.firewall_policies_client.create_firewall_policy(
            name=data_utils.rand_name("fw-policy"),
            **kwargs)
        fw_policy = body['firewall_policy']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.firewall_policies_client.delete_firewall_policy,
                        fw_policy['id'])
        return fw_policy

    def create_firewall_group(self, **kwargs):
        body = self.firewall_groups_client.create_firewall_group(
            name=data_utils.rand_name("fwg"),
            **kwargs)
        fwg = body['firewall_group']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.delete_firewall_group_and_wait,
                        fwg['id'])
        return fwg

    def delete_firewall_group_and_wait(self, firewall_group_id):
        self.firewall_groups_client.delete_firewall_group(firewall_group_id)
        self._wait_firewall_group_while(firewall_group_id,
                                        [nl_constants.PENDING_DELETE],
                                        not_found_ok=True)

    def insert_firewall_rule_in_policy_and_wait(self,
                                                firewall_group_id,
                                                firewall_policy_id,
                                                firewall_rule_id, **kwargs):
        self.firewall_policies_client.insert_firewall_rule_in_policy(
            firewall_policy_id=firewall_policy_id,
            firewall_rule_id=firewall_rule_id,
            **kwargs)
        self.addCleanup(
            self._call_and_ignore_exceptions,
            (lib_exc.NotFound, lib_exc.BadRequest),
            self.remove_firewall_rule_from_policy_and_wait,
            firewall_group_id=firewall_group_id,
            firewall_policy_id=firewall_policy_id,
            firewall_rule_id=firewall_rule_id)
        self._wait_firewall_group_ready(firewall_group_id)

    def remove_firewall_rule_from_policy_and_wait(self,
                                                  firewall_group_id,
                                                  firewall_policy_id,
                                                  firewall_rule_id):
        self.firewall_policies_client.remove_firewall_rule_from_policy(
            firewall_policy_id=firewall_policy_id,
            firewall_rule_id=firewall_rule_id)
        self._wait_firewall_group_ready(firewall_group_id)

    def update_firewall_group_and_wait(self, firewall_group_id, **kwargs):
        self.firewall_groups_client.update_firewall_group(
            firewall_group_id, **kwargs)
        self._wait_firewall_group_ready(firewall_group_id)

    @staticmethod
    def _call_and_ignore_exceptions(exc_list, func, *args, **kwargs):
        """Call the given function and pass if a given exception is raised."""

        try:
            return func(*args, **kwargs)
        except exc_list:
            pass

    def _wait_firewall_group_ready(self, firewall_group_id):
        self._wait_firewall_group_while(firewall_group_id,
                                        [nl_constants.PENDING_CREATE,
                                        nl_constants.PENDING_UPDATE])

    def _wait_firewall_group_while(self, firewall_group_id, statuses,
                                   not_found_ok=False):
        start = int(time.time())
        if not_found_ok:
            expected_exceptions = (lib_exc.NotFound)
        else:
            expected_exceptions = ()
        while True:
            try:
                fwg = self.firewall_groups_client.show_firewall_group(
                    firewall_group_id)
            except expected_exceptions:
                break
            status = fwg['firewall_group']['status']
            if status not in statuses:
                break
            if (int(time.time()) - start >=
                self.firewall_groups_client.build_timeout):
                msg = ("Firewall Group %(firewall_group)s failed to reach "
                       "non PENDING status (current %(status)s)") % {
                    "firewall_group": firewall_group_id,
                    "status": status,
                }
                raise exceptions.TimeoutException(msg)
            time.sleep(1)



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_tempest_plugin.common import tempest_fixtures
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base


class AgentManagementTestJSON(base.BaseAdminNetworkTest):

    required_extensions = ['agent']

    @classmethod
    def resource_setup(cls):
        super(AgentManagementTestJSON, cls).resource_setup()
        body = cls.admin_client.list_agents()
        agents = body['agents']
        cls.agent = agents[0]  # don't modify this agent

    @decorators.idempotent_id('9c80f04d-11f3-44a4-8738-ed2f879b0ff4')
    def test_list_agent(self):
        body = self.admin_client.list_agents()
        agents = body['agents']
        # Heartbeats must be excluded from comparison
        self.agent.pop('heartbeat_timestamp', None)
        self.agent.pop('configurations', None)
        # Exclude alive as it can happen that when testclass'
        # resource_setup executed the selected agent is not up
        self.agent.pop('alive', None)
        for agent in agents:
            agent.pop('heartbeat_timestamp', None)
            agent.pop('configurations', None)
            agent.pop('alive', None)
        self.assertIn(self.agent, agents)

    @decorators.idempotent_id('e335be47-b9a1-46fd-be30-0874c0b751e6')
    def test_list_agents_non_admin(self):
        body = self.client.list_agents()
        self.assertEqual(len(body["agents"]), 0)

    @decorators.idempotent_id('869bc8e8-0fda-4a30-9b71-f8a7cf58ca9f')
    def test_show_agent(self):
        body = self.admin_client.show_agent(self.agent['id'])
        agent = body['agent']
        self.assertEqual(agent['id'], self.agent['id'])

    @decorators.idempotent_id('371dfc5b-55b9-4cb5-ac82-c40eadaac941')
    def test_update_agent_status(self):
        origin_status = self.agent['admin_state_up']
        # Try to update the 'admin_state_up' to the original
        # one to avoid the negative effect.
        agent_status = {'admin_state_up': origin_status}
        body = self.admin_client.update_agent(agent_id=self.agent['id'],
                                              agent_info=agent_status)
        updated_status = body['agent']['admin_state_up']
        self.assertEqual(origin_status, updated_status)

    @decorators.idempotent_id('68a94a14-1243-46e6-83bf-157627e31556')
    def test_update_agent_description(self):
        agents = self.admin_client.list_agents()['agents']
        dyn_agent = self._select_one_agent_for_update(agents)

        self.useFixture(tempest_fixtures.LockFixture('agent_description'))
        description = 'description for update agent.'
        agent_description = {'description': description}
        body = self.admin_client.update_agent(agent_id=dyn_agent['id'],
                                              agent_info=agent_description)
        self.addCleanup(self._restore_agent, dyn_agent)
        updated_description = body['agent']['description']
        self.assertEqual(updated_description, description)

    def _restore_agent(self, dyn_agent):
        """Restore the agent description after update test."""
        description = dyn_agent['description']
        origin_agent = {'description': description}
        self.admin_client.update_agent(agent_id=dyn_agent['id'],
                                       agent_info=origin_agent)

    def _select_one_agent_for_update(self, agents):
        """Return one agent that is not the one selected at resource_setup"""
        for agent in agents:
            if self.agent['id'] != agent['id']:
                return agent
        raise self.skipException("This test requires at least two agents.")

    @decorators.idempotent_id('b33af888-b6ac-4e68-a0ca-0444c2696cf9')
    def test_delete_agent_negative(self):
        non_existent_id = data_utils.rand_uuid()
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_client.delete_agent, non_existent_id)
